UNIT KSSFoldr;
{+F
---------------------------------------------------------------------------
  File        - FSSFOLDR.PAS
  Purpose     - Create a component around the FrmFoldr (select a folder).
  Author      - Keith S. Brown
  Copyright    2000 by KSS (Keith S. Brown)
  Revised     - 2000.08.24(KSB) Created original.
---------------------------------------------------------------------------}
INTERFACE
USES
  Classes,  Graphics, Forms;

TYPE
  TKSSselectFolderDialog= CLASS(TComponent)
  PRIVATE
    FsCaption : STRING;
    FsFolder  : STRING;
  PUBLIC
    CONSTRUCTOR Create(aoOwner:TComponent); OVERRIDE;
    DESTRUCTOR  Destroy; OVERRIDE;
    FUNCTION    Execute:BOOLEAN;
  PUBLISHED
    PROPERTY    Caption:STRING   READ FsCaption WRITE FsCaption;
    PROPERTY    Directory:STRING READ FsFolder  WRITE FsFolder;
  END {CLASS};

{}PROCEDURE   Register;
              {=================================================}
IMPLEMENTATION
USES
  Windows,
  Controls,
  SysUtils,
  FrmFoldr;


{}CONSTRUCTOR TKSSselectFolderDialog.Create(aoOwner:TComponent);
{+M
---------------------------------------------------------------------------
  Purpose     - Create any additional components.
---------------------------------------------------------------------------}
  BEGIN
    INHERITED Create(aoOwner);
{}END {Create};




{}DESTRUCTOR  TKSSselectFolderDialog.Destroy;
{+M
---------------------------------------------------------------------------
  Purpose     - Release any allocated memory.
---------------------------------------------------------------------------}
  BEGIN
    INHERITED Destroy;
{}END {Destroy};




{}FUNCTION    TKSSselectFolderDialog.Execute:BOOLEAN;
{+M
---------------------------------------------------------------------------
  Purpose     - Create the dialog, set the properties & return the result.
---------------------------------------------------------------------------}
  VAR
    lsPath    : STRING;
  BEGIN
    WITH TdlgSelectFolder.Create(application) DO BEGIN
      TRY
        Caption := FsCaption;
        IF FsFolder <> '' THEN BEGIN
          lsPath := ExtractFileDrive(FsFolder);
          IF lsPath <> '' THEN BEGIN
            cbxDrives.Drive := lsPath[1];
          END {IF};

          lbxFolders.Directory := FsFolder;
        END {IF};

        result := ShowModal = idOk;
        IF result THEN BEGIN
          FsFolder := lblFolder.Caption;
        END {IF};
      FINALLY
        Release
      END {TRY};
    END {WITH};
{}END {Execute};




{}PROCEDURE   Register;
{+H
---------------------------------------------------------------------------
  Purpose     - Register the dialog.
---------------------------------------------------------------------------}
  BEGIN
    RegisterComponents('Dialogs',[TKSSselectFolderDialog]);
{}END {Register};


END {KSSFoldr}.
