UNIT FrmDeflts;
{+F
---------------------------------------------------------------------------
  File        - FRMDEFAULTS.PAS
  Purpose     - Set the global defaults.
  Author      - Keith S. Brown
  Copyright    2000 by KSS  http://www.ghg.net/chairouchu
  Revised     - 2000.08.22(KSB)
---------------------------------------------------------------------------}
INTERFACE
USES
  Windows,  Messages, SysUtils, Classes,  Graphics, Controls, Forms,    Dialogs,
  StdCtrls, Buttons,  ExtCtrls, KSSFoldr;

TYPE
  TdlgDefaults= CLASS(TForm)
    btnCancel : TBitBtn;
    btnGetRTL : TSpeedButton;
    btnGetVCL : TSpeedButton;
    btnGetView: TSpeedButton;
    btnNotePad: TRadioButton;
    btnOk     : TBitBtn;
    btnOther  : TRadioButton;
    btnSearch : TSpeedButton;
    btnWordPad: TRadioButton;
    dlgFolder : TKSSselectFolderDialog;
    dlgOpen   : TOpenDialog;
    edtRTL    : TEdit;
    edtSearch : TEdit;
    edtVCL    : TEdit;
    edtViewer : TEdit;
    gbxPaths  : TGroupBox;
    lblResults: TLabel;
    lblRTL    : TLabel;
    lblSearch : TLabel;
    lblVCL    : TLabel;
    pnlEditor : TPanel;
    PROCEDURE   DoClose(aoSender: TObject);
    PROCEDURE   DoSave(aoSender: TObject);
    PROCEDURE   FormCreate(aoSender: TObject);
    PROCEDURE   GetPathClick(aoSender: TObject);
    PROCEDURE   GetViewerClick(aoSender: TObject);
  END {CLASS};

              {=================================================}
IMPLEMENTATION
{$R *.DFM}
USES
  KSString,
  LstGlobal;


{}PROCEDURE   TdlgDefaults.DoClose(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     - Close the form.
---------------------------------------------------------------------------}
  BEGIN
    Close;
{}END {DoClose};




{}PROCEDURE   TdlgDefaults.DoSave(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     - Save all values.
---------------------------------------------------------------------------}
  BEGIN
    IF btnWordPad.Checked THEN BEGIN
      geEditorIs := eiWordPad;
    END ELSE
    IF btnNotePad.Checked THEN BEGIN
      geEditorIs := eiNotePad;
    END ELSE BEGIN
      geEditorIs := eiOther;
    END {IF};

    gsDelphi_RTL := AddBackSlash(edtRTL.Text,'');
    gsDelphi_VCL := AddBackSlash(edtVCL.Text,'');
    gsEditorPath := edtViewer.Text;
    gsSearchPath := edtSearch.Text;

    SaveControlPos(self);
    SaveConfiguration;
    Close;
{}END {DoSave};




{}PROCEDURE   TdlgDefaults.FormCreate(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     - Read the values and set the displayed values accordingly.
---------------------------------------------------------------------------}
  VAR
    liHeight  : INTEGER;
  BEGIN
    liHeight := Height;
    ReadControlPos(self);
    Height := liHeight;
    ReadConfiguration;

    edtRTL.Text   := gsDelphi_RTL;
    edtSearch.Text:= gsSearchPath;
    edtVCL.Text   := gsDelphi_VCL;
    edtViewer.Text:= gsEditorPath;
    CASE geEditorIs OF
      eiNotePad : btnNotePad.Checked := TRUE;
      eiOther   : btnOther.Checked   := TRUE;
      ELSE        btnWordPad.Checked := TRUE;
    END {CASE};
{}END {FormCreate};




{}PROCEDURE   TdlgDefaults.GetPathClick(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     - Get the paths for the RTL & VCL.
  Revised     - 2000.08.25 (KSB) Restored original directory after finished.
---------------------------------------------------------------------------}
  VAR
    lsHomePath: STRING;
  BEGIN
    IF aoSender IS TSpeedButton THEN BEGIN
      lsHomePath := GetCurrentDir;
      IF TSpeedButton(aoSender).Name = btnGetRTL.Name THEN BEGIN
        dlgFolder.Directory := edtRTL.Text;
        IF dlgFolder.Execute THEN BEGIN
          edtRTL.Text := AddBackSlash(dlgFolder.Directory,'');
        END {IF};
      END ELSE
      IF TSpeedButton(aoSender).Name = btnGetVCL.Name THEN BEGIN
        dlgFolder.Directory := edtVCL.Text;
        IF dlgFolder.Execute THEN BEGIN
          edtVCL.Text := AddBackSlash(dlgFolder.Directory,'');
        END {IF};
      END ELSE
      IF TSpeedButton(aoSender).Name = btnSearch.Name THEN BEGIN
        IF dlgFolder.Execute THEN BEGIN
          dlgFolder.Directory := AddBackSlash(dlgFolder.Directory,'');
          IF Length(edtSearch.Text) > 0 THEN BEGIN
            edtSearch.Text := edtSearch.Text + ';' + dlgFolder.Directory;
          END ELSE BEGIN
            edtSearch.Text := AddBackSlash(dlgFolder.Directory,'');
          END {IF};
        END {IF};
      END {IF};

      SetCurrentDir(lsHomePath);
    END {IF};
{}END {GetPathClick};




{}PROCEDURE   TdlgDefaults.GetViewerClick(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     - Get a path for the report viewer (generally an editor).
---------------------------------------------------------------------------}
  BEGIN
    IF aoSender IS TSpeedButton THEN BEGIN
      IF TSpeedButton(aoSender).Name = btnGetView.Name THEN BEGIN
        dlgOpen.FileName := edtViewer.Text;
        IF dlgOpen.Execute THEN BEGIN
          edtViewer.Text := dlgOpen.FileName;
        END {IF};
      END {IF};
    END {IF};
{}END {GetViewerClick};


END {FrmDeflts}.
