UNIT FrmMain;
{+F
---------------------------------------------------------------------------
  File        - FRMMAIN.PAS
  Purpose     - GUI line counter.
  Author      - Keith S. Brown
  Copyright    2000 by KSS (Keith S. Brown)
  Revised     - 2000.08.18(KSB) Wrote initial based on W32Lcnt.
              - 2000.08.22(KSB) Modified Unit list/tree/dependency logic.
              - 2000.08.23(KSB) Added default dialog to set paths & viewer.
              - 2000.08.24(KSB) Added About box & other dialogs.
---------------------------------------------------------------------------}
INTERFACE
USES
  Windows,  Messages, SysUtils, Classes,  Graphics, Forms,    Dialogs,  Controls,
  StdCtrls, Buttons,  ExtCtrls, Menus;

TYPE
  TdlgMain    = CLASS(TForm)
    btnCancel : TButton;
    btnDelAll : TSpeedButton;
    btnExAll  : TSpeedButton;
    btnExclude: TSpeedButton;
    btnIncAll : TSpeedButton;
    btnInclude: TSpeedButton;
    btnLCnt   : TButton;
    btnOptions: TBitBtn;
    btnSource : TBitBtn;
    btnUDpnd  : TButton;
    btnUList  : TButton;
    btnUTree  : TButton;
    chkReuseIt: TCheckBox;
    chkRTL    : TCheckBox;
    chkSearchP: TCheckBox;
    chkVCL    : TCheckBox;
    dlgOpen   : TOpenDialog;
    lblDst    : TLabel;
    lblSrc    : TLabel;
    lstDst    : TListBox;
    lstSrc    : TListBox;
    pnlButton : TPanel;
    pnlDst    : TPanel;
    pnlCtrls  : TPanel;
    pnlLeft   : TPanel;
    pnlOptions: TPanel;
    pnlSrc    : TPanel;
    mniAbout  : TMenuItem;
    mniN1     : TMenuItem;
    mniOptions: TMenuItem;
    mnuPopup  : TPopupMenu;
    splHorizon: TSplitter;

    PROCEDURE   DoAboutClick(aoSender: TObject);
    PROCEDURE   DoCancelClick(aoSender: TObject);
    PROCEDURE   DoCheckBoxClick(aoSender: TObject);
    PROCEDURE   DoCountLines(aoSender: TObject);
    PROCEDURE   DoDelAllClick(aoSender: TObject);
    PROCEDURE   DoExcAllClick(aoSender: TObject);
    PROCEDURE   DoExcludeClick(aoSender: TObject);
    PROCEDURE   DoIncAllClick(aoSender: TObject);
    PROCEDURE   DoIncludeClick(aoSender: TObject);
    PROCEDURE   DoOptionsClick(aoSender: TObject);
    PROCEDURE   DoSourceClick(aoSender: TObject);
    PROCEDURE   DoUnitDependancyClick(aoSender: TObject);
    PROCEDURE   DoUnitList(aoSender: TObject);
    PROCEDURE   DoUnitTreeClick(aoSender: TObject);
    PROCEDURE   FormKeyDown(aoSender: TObject; VAR aiKey: WORD; asetShift: TShiftState);
    PROCEDURE   FormResize(aoSender: TObject);
    PROCEDURE   FormShow(aoSender: TObject);
    PROCEDURE   lstMouseDown(aoSender: TObject; aeButton: TMouseButton; asetShift: TShiftState; aiX, aiY: INTEGER);
    PROCEDURE   pnlLeftResize(aoSender: TObject);
    PROCEDURE   pnlSrcResize(aoSender: TObject);
  PRIVATE
    PROCEDURE   DisplayFile(asFileName:STRING);
    PROCEDURE   JustOneProcessFile;
    PROCEDURE   OptionsLoad;
    FUNCTION    ReportFileOk(VAR asFileName:STRING):BOOLEAN;
    FUNCTION    ReuseList:BOOLEAN;
    PROCEDURE   UpdateReuseIt;
  PUBLIC
    FUNCTION    GetFirstSelection(aoList: TCustomListBox): INTEGER;
    PROCEDURE   MoveSelected(aoList: TCustomListBox; aoItems: TStrings);
    PROCEDURE   SetButtons;
    PROCEDURE   SetItem(aoList: TListBox; aiIndex: INTEGER);
  END {CLASS};

VAR
  dlgMain: TdlgMain;
              {=================================================}
IMPLEMENTATION
USES
  IniFiles,
  LCntEngn,
  UnitTree,
  UnitDpnd,
  FrmDeflts,
  LstGlobal,
  FrmExists,
  KSSAbout;

{$R *.DFM}
CONST
  ciMinFormCWidth  = 455;
  ciMinFormCHeight = 255;
  csBuildDate      = 'Aug 2000';
VAR
  usVersion        : STRING;



{}PROCEDURE   TdlgMain.DisplayFile(asFileName:STRING);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    laWinDir  : ARRAY[0..MAX_PATH]OF CHAR;
    li        : INTEGER;
    loINI     : TIniFile;
    ls        : STRING;
    lsPath    : STRING;
  BEGIN
              {display the report using word pad if that option is checked...}
    CASE geEditorIs OF
      eiWordPad : BEGIN
              {get the wordpad path from the windows ini file...}
        GetWindowsDirectory(laWinDir,MAX_PATH);
        ls := StrPas(laWinDir);
        loINI := TIniFile.Create(ls+'\Win.ini');
        lsPath := loINI.ReadString('embedding','Wordpad.Document.1','');
        loINI.Free;

              {the line in the ini file is of the following form:
               Wordpad.Document.1=WordPad Document,WordPad Document,C:\PROGRA~1\ACCESS~1\WORDPAD.EXE,picture
               we want just the path, which if it exists should be between the 2nd & 3rd commas.}
        IF lsPath <> '' THEN BEGIN
          li := 0;
          WHILE Pos(',',lsPath) > 0 DO BEGIN
            Inc(li);
            CASE li OF
              1,
              2 : Delete(lsPath,1,Pos(',',lsPath));
              3 : Delete(lsPath,Pos(',',lsPath),Length(ls)-Pos(',',ls)+1);
            END {CASE};
          END {IF};
        END {IF};

              {If wordpad is not found, then default to NotePad...}
        IF NOT FileExists(lsPath) THEN BEGIN
          lsPath := 'NotePad ';
        END {IF};
      END {eiWordPad};

      eiNotePad : lsPath := 'NotePad ';
      eiOther   : lsPath := gsEditorPath+' ';
    END {CASE};

              {now display the report using NotePad or WordPad as appropriate...}
    lsPath := lsPath+' '+asFileName;
    IF WinExec(PChar(lsPath), SW_SHOWNORMAL) < 32 THEN BEGIN
      MessageDlg('Failed to execute '+lsPath,mtError,[mbOk],0);
    END {IF};
{}END {DisplayFile};




{}PROCEDURE   TdlgMain.DoAboutClick(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     - Show the about box.
---------------------------------------------------------------------------}
  BEGIN
    WITH TAboutBox.Create(self) DO BEGIN
      CopyrightYear := 2000;
      ProgramName := 'W32 Delphi Tool';
      ShowModal;
      usVersion := Version;
      Release;
    END {WITH};
{}END {DoAboutClick};




{}PROCEDURE   TdlgMain.DoCancelClick(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     - Save the options and form state before closing.
  Revised     - 2000.08.17(KSB) Save form position.
---------------------------------------------------------------------------}
  BEGIN
    SaveControlPos(self);
    SaveConfiguration;
    Close;
{}END {DoCancelClick};




{}PROCEDURE   TdlgMain.DoCheckBoxClick(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    IF aoSender IS TCheckBox THEN BEGIN
      IF TCheckBox(aoSender).Name = chkRTL.Name THEN BEGIN
        gbUseRTL := chkRTL.Checked;
      END ELSE
      IF TCheckBox(aoSender).Name = chkVCL.Name THEN BEGIN
        gbUseVCL := chkVCL.Checked;
      END ELSE
      IF TCheckBox(aoSender).Name = chkSearchP.Name THEN BEGIN
        gbUseSearchP := chkSearchP.Checked;
      END {IF};
    END {IF};
{}END {DoCheckBoxClick};




{}PROCEDURE   TdlgMain.DoCountLines(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     - Activate the line count engine.
  Revised     - 2000.08.18(KSB) Modified from W32Lcnt.
---------------------------------------------------------------------------}
  VAR
    loLcnt    : TLcntEngine;
    lTxt      : TEXTFILE;

    
{}{}PROCEDURE Wrt_List(aoStrings:TStrings);
    VAR
      li      : INTEGER;
    BEGIN
      AssignFile(lTxt,loLcnt.FileName);
      Rewrite(lTxt);
      TRY
        FOR li := 0 TO aoStrings.Count-1 DO BEGIN
          WriteLn(lTxt,aoStrings[li]);
        END {FOR};
      FINALLY
        CloseFile(lTxt);
      END {TRY};
{}{}END {Wrt_List};


  VAR
    loLines   : TStringList;
    lsReport  : STRING;
  BEGIN
    IF lstDst.Items.Count = 0 THEN BEGIN
      DoSourceClick(aoSender);
      DoIncAllClick(aoSender);
    END {IF};

    IF lstDst.Items.Count = 0 THEN BEGIN
      Exit;
    END {IF};

    lsReport := lblSrc.Caption+gsReportFile;
    IF NOT ReportFileOk(lsReport) THEN BEGIN
      Exit;
    END {IF};

    loLcnt := TLcntEngine.Create;
    TRY
      loLcnt.FileName := lblSrc.Caption+'W32LCnt.Files';
      IF ReuseList THEN BEGIN
        Wrt_List(goFileList);
      END ELSE BEGIN
        lstDst.Sorted := TRUE;
        Wrt_List(lstDst.Items);
      END {IF};

      screen.Cursor := crHourGlass;
      loLcnt.IsFileList := TRUE;
      loLcnt.IsDetailedReport := TRUE;
      IF loLcnt.Execute THEN BEGIN
        IF FileExists(lsReport) THEN BEGIN
          loLines := TStringList.Create;
          TRY
            loLines.LoadFromFile(lsReport);
            loLines.AddStrings(loLcnt.Report);
            loLines.SaveToFile(lsReport);
          FINALLY
            FreeAndNil(loLines);
          END {TRY};
        END ELSE BEGIN
          loLcnt.Report.SaveToFile(lsReport);
        END {IF};

        DisplayFile(lsReport);
        UpdateReuseIt;
      END {IF};
    FINALLY
      screen.Cursor := crDefault;
      FreeAndNil(loLcnt);
    END {TRY};
{}END {DoCountLines};




{}PROCEDURE   TdlgMain.DoDelAllClick(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    Units_ClearTree;
    lstSrc.Clear;
    lstDst.Clear;
    SetButtons;
{}END {DoDelAllClick};




{}PROCEDURE   TdlgMain.DoExcAllClick(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    li        : INTEGER;
  BEGIN
    FOR li := 0 TO lstDst.Items.Count - 1 DO BEGIN
      lstSrc.Items.AddObject(lstDst.Items[li], lstDst.Items.Objects[li]);
    END {FOR};

    lstDst.Items.Clear;
    SetItem(lstDst, 0);
    Units_ClearTree;
{}END {DoExcAll};




{}PROCEDURE   TdlgMain.DoExcludeClick(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    liIndex   : INTEGER;
  BEGIN
    IF lstDst.Items.Count > 0 THEN BEGIN
      liIndex := GetFirstSelection(lstDst);
      MoveSelected(lstDst, lstSrc.Items);
      SetItem(lstDst, liIndex);
      IF Assigned(aoSender) THEN BEGIN
        Units_ClearTree;
      END {IF};
    END {IF};
{}END {DoExcludeClick};




{}PROCEDURE   TdlgMain.DoIncAllClick(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    li        : INTEGER;
  BEGIN
    FOR li := 0 TO lstSrc.Items.Count - 1 DO BEGIN
      lstDst.Items.AddObject(lstSrc.Items[li], lstSrc.Items.Objects[li]);
    END {FOR};

    lstSrc.Items.Clear;
    SetItem(lstSrc, 0);
    IF Assigned(aoSender) THEN BEGIN
      Units_ClearTree;
    END {IF};
{}END {DoIncAllClick};




{}PROCEDURE   TdlgMain.DoIncludeClick(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    liIndex   : INTEGER;
  BEGIN
    IF lstSrc.Items.Count > 0 THEN BEGIN
      liIndex := GetFirstSelection(lstSrc);
      MoveSelected(lstSrc, lstDst.Items);
      SetItem(lstSrc, liIndex);
      IF Assigned(aoSender) THEN BEGIN
        Units_ClearTree;
      END {IF};
    END {IF};
{}END {DoIncludeClick};




{}PROCEDURE   TdlgMain.DoOptionsClick(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    WITH TdlgDefaults.Create(self) DO BEGIN
      ShowModal;
      Release
    END {WITH};
{}END {DoOptionsClick};




{}PROCEDURE   TdlgMain.DoSourceClick(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    li        : INTEGER;
  BEGIN
    WITH dlgOpen DO BEGIN
      Filter := 'Projects (*.dpr)|*.dpr|Units (*.pas)|*.pas|Include (*.in?)|*.in?|All Source Files (*.dpr; *.pas; *.in?)|*.dpr;*.pas;*.in*';
      FilterIndex := 4;
      Options := [ofReadOnly,ofHideReadOnly,ofAllowMultiSelect,ofPathMustExist,ofFileMustExist,ofEnableSizing];
      Title := 'Select Source Files';
      IF Execute THEN BEGIN
        lstSrc.Clear;
        lblSrc.Caption := ExtractFilePath(FileName);
        FOR li := 0 TO Files.Count - 1 DO BEGIN
          lstSrc.Items.Add(Files.Strings[li]);  { next file in Files property }
        END {FOR};
      END {IF};

      IF lstSrc.Items.Count > 0 THEN BEGIN
        btnIncAll.Enabled := TRUE;
        btnInclude.Enabled:= TRUE;
      END {IF};
    END {WITH};
{}END {DoSourceClick};




{}PROCEDURE   TdlgMain.DoUnitDependancyClick(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    lsReport  : STRING;
  BEGIN
    IF lstDst.Items.Count = 0 THEN BEGIN
      DoSourceClick(aoSender);
      DoIncAllClick(aoSender);
    END {IF};

    JustOneProcessFile;
    IF lstDst.Items.Count = 0 THEN BEGIN
      Exit;
    END {IF};

    lsReport := lblSrc.Caption+gsReportFile;
    IF NOT ReportFileOk(lsReport) THEN BEGIN
      Exit;
    END {IF};

    IF NOT ReuseList THEN BEGIN
      application.ProcessMessages;
      screen.Cursor := crHourGlass;
      TRY
        Units_BuildTree(lstDst.Items[0]);
      FINALLY
        screen.Cursor := crDefault;
      END {TRY};
    END {IF};

    Write_UnitDpnd(lstDst.Items[0],lsReport);
    DisplayFile(lsReport);
    UpdateReuseIt;
{}END {DoUnitDependancyClick};




{}PROCEDURE   TdlgMain.DoUnitList(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    lsReport  : STRING;
  BEGIN
    IF lstDst.Items.Count = 0 THEN BEGIN
      DoSourceClick(aoSender);
      DoIncAllClick(aoSender);
    END {IF};

    JustOneProcessFile;
    IF lstDst.Items.Count = 0 THEN BEGIN
      Exit;
    END {IF};

    lsReport := lblSrc.Caption+gsReportFile;
    IF NOT ReportFileOk(lsReport) THEN BEGIN
      Exit;
    END {IF};

    IF NOT ReuseList THEN BEGIN
      application.ProcessMessages;
      screen.Cursor := crHourGlass;
      TRY
        Units_BuildTree(lstDst.Items[0]);
      FINALLY
        screen.Cursor := crDefault;
      END {TRY};
    END {IF};

    Write_UnitList(lsReport);
    DisplayFile(lsReport);
    UpdateReuseIt;
{}END {DoUnitList};




{}PROCEDURE   TdlgMain.DoUnitTreeClick(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    lsReport  : STRING;
  BEGIN
    IF lstDst.Items.Count = 0 THEN BEGIN
      DoSourceClick(aoSender);
      DoIncAllClick(aoSender);
    END {IF};

    JustOneProcessFile;
    IF lstDst.Items.Count = 0 THEN BEGIN
      Exit;
    END {IF};

    lsReport := lblSrc.Caption+gsReportFile;
    IF NOT ReportFileOk(lsReport) THEN BEGIN
      Exit;
    END {IF};

    IF NOT ReuseList THEN BEGIN
      application.ProcessMessages;
      screen.Cursor := crHourGlass;
      TRY
        Units_BuildTree(lstDst.Items[0]);
      FINALLY
        screen.Cursor := crDefault;
      END {TRY};
    END {IF};

    Write_UnitTree(lsReport);
    DisplayFile(lsReport);
    UpdateReuseIt;
{}END {DoUnitTreeClick};




{}PROCEDURE TdlgMain.FormKeyDown(aoSender: TObject; VAR aiKey: WORD; asetShift: TShiftState);
{+M
---------------------------------------------------------------------------
  Purpose     - Provide an Alt-X short cut to close.
---------------------------------------------------------------------------}
  BEGIN
    IF (ssAlt IN asetShift) AND (aiKey = Ord('X')) THEN BEGIN
      DoCancelClick(aoSender);
    END {IF};
{}END {FormKeyDown};




{}PROCEDURE   TdlgMain.FormResize(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  CONST
    ciLRmargin= 4;
  BEGIN
    IF clientWidth < ciMinFormCWidth THEN BEGIN
      clientWidth := ciMinFormCWidth;
    END {IF};

    IF clientHeight < ciMinFormCHeight THEN BEGIN
      clientHeight := ciMinFormCHeight;
    END {IF};

    btnOptions.Left := pnlButton.Width - btnOptions.Width;

    WITH btnCancel DO BEGIN
      Width:= pnlDst.Width;
      Left := pnlDst.Left;
      IF Left < (btnSource.Left+btnSource.Width) THEN BEGIN
        Left := (btnSource.Left+btnSource.Width);
      END {IF};

      IF (Width+Left) > (pnlButton.Width-ciLRmargin) THEN BEGIN
        Width := self.ClientWidth - Left - ciLRmargin;
      END {IF};
    END {WITH};
{}END {FormResize};




{}PROCEDURE   TdlgMain.FormShow(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    OptionsLoad;
    SetButtons;
    FormResize(aoSender);
    DoAboutClick(aoSender);
    Caption := 'W32 Delphi Tool  V:'+usVersion+'  '+csBuildDate;
{}END {FormShow};




{}FUNCTION    TdlgMain.GetFirstSelection(aoList: TCustomListBox): INTEGER;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    FOR Result := 0 TO aoList.Items.Count - 1 DO BEGIN
      IF aoList.Selected[Result] THEN BEGIN
        Exit;
      END {IF};
    END {FOR};

    Result := LB_ERR;
{}END {GetFirstSelection};




{}PROCEDURE   TdlgMain.JustOneProcessFile;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
{}{}FUNCTION  IsDpr(asFileName:STRING):BOOLEAN;
    BEGIN
      result := UpperCase(ExtractFileExt(asFileName)) = '.DPR';
{}{}END {IsDpr};


  VAR
    liDpr     : INTEGER;
  BEGIN
    liDpr := 0;
    IF lstDst.Items.Count > 1 THEN BEGIN
      lstDst.ItemIndex := 0;
      REPEAT
        IF IsDpr(lstDst.Items[lstDst.ItemIndex]) THEN BEGIN
          Inc(liDpr);
        END {IF};

        lstDst.ItemIndex := lstDst.ItemIndex + 1;
      UNTIL lstDst.ItemIndex = lstDst.Items.Count -1;

      lstDst.ItemIndex := 0;
      WHILE lstDst.Items.Count > 1 DO BEGIN
        IF (liDpr > 0) AND IsDpr(lstDst.Items[lstDst.ItemIndex]) THEN BEGIN
          liDpr := 0;
          lstDst.ItemIndex := lstDst.ItemIndex + 1;
        END ELSE BEGIN
          lstDst.ItemIndex := 1;
        END {IF};

        DoExcludeClick(NIL);
      END {WHILE};
    END {IF};
{}END {JustOneProcessFile};




{}PROCEDURE   TdlgMain.lstMouseDown(aoSender: TObject; aeButton: TMouseButton; asetShift: TShiftState; aiX, aiY: INTEGER);
  BEGIN
    IF aeButton = mbLeft THEN BEGIN
      IF aoSender IS TListBox THEN BEGIN
        IF TListBox(aoSender).Name = lstSrc.Name THEN BEGIN
          DoSourceClick(aoSender);
        END ELSE
        IF TListBox(aoSender).Name = lstDst.Name THEN BEGIN
          DoIncludeClick(aoSender);
        END {IF};
      END {IF};
    END {IF};
{}END {lstMouseDown};




{}PROCEDURE   TdlgMain.MoveSelected(aoList: TCustomListBox; aoItems: TStrings);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    li        : INTEGER;
  BEGIN
    FOR li := aoList.Items.Count - 1 DOWNTO 0 DO BEGIN
      IF aoList.Selected[li] THEN BEGIN
        aoItems.AddObject(aoList.Items[li], aoList.Items.Objects[li]);
        aoList.Items.Delete(li);
      END {IF};
    END {FOR};
{}END {MoveSelected};





{}PROCEDURE   TdlgMain.OptionsLoad;
{+M
---------------------------------------------------------------------------
  Purpose     - Restore the options from the previous run.
  Revised     - 2000.08.17(KSB) Read saved form position.
---------------------------------------------------------------------------}
  BEGIN
    ReadControlPos(self);
    ReadConfiguration;
    chkRTL.Checked := gbUseRTL;
    chkVCL.Checked := gbUseVCL;
    chkSearchP.Checked:= gbUseSearchP;
{}END {OptionsLoad};




{}PROCEDURE   TdlgMain.pnlLeftResize(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    IF pnlLeft.Width < pnlCtrls.Width THEN BEGIN
      pnlLeft.Width := pnlCtrls.Width;
    END {IF};
{}END {pnlLeftResize};




{}PROCEDURE   TdlgMain.pnlSrcResize(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    lblSrc.Width := pnlSrc.Width-4;
{}END {pnlSrcResize};




{}FUNCTION    TdlgMain.ReportFileOk(VAR asFileName:STRING):BOOLEAN;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    liResult  : INTEGER;
    lTxt      : TEXTFILE;
  BEGIN
    IF FileExists(asFileName) THEN BEGIN
      WITH TdlgFileExists.Create(self) DO BEGIN
        TRY
          IF chkReuseIt.Checked THEN BEGIN
            DefaultToAppend;
          END ELSE BEGIN
            DefaultToOverwrite;
          END {IF};

          lblMsg.Caption := 'The report file "'+asFileName+'" already exists. Either Append, Overwrite, Select a new file or Cancel.';
          liResult := ShowModal;
        FINALLY
          Release;
        END {WITH};
      END {WITH};
    END ELSE BEGIN
      liResult := mrNo;
    END {IF};

    CASE liResult OF
      mrCancel : result := FALSE;
      mrYes    : result := TRUE;
      mrNo     : BEGIN
        result := TRUE;
        AssignFile(lTxt,asFileName);
        Rewrite(lTxt);
        CloseFile(lTxt);
      END {mrNo};

      mrRetry  : BEGIN
        WITH dlgOpen DO BEGIN
          Filter := 'Text Files (*.txt)|*.txt|Report Files (*.rpt)|*.rpt|All Files (*.*)|*.*';
          FilterIndex := 1;
          Options := [ofPathMustExist,ofEnableSizing];
          Title := 'Select Report File';
          FileName := asFileName;
          IF Execute THEN BEGIN
            asFilename := FileName;
            result := ReportFileOk(asFileName);
          END {IF};
        END {WITH};
      END {mrNo};
    END {CASE};
{}END {ReportFileOk};




{}FUNCTION    TdlgMain.ReuseList:BOOLEAN;
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    IF (goFileList.Count > 0) THEN BEGIN
      IF chkReuseIt.Checked THEN BEGIN
        result := TRUE;
      END ELSE BEGIN
        result := MessageDlg('Reuse existing list of units?',mtConfirmation,[mbYes,mbNo],0) = mrYes;
      END {IF};
    END ELSE BEGIN
      result := FALSE;
    END {IF};
{}END {ReuseList};




{}PROCEDURE   TdlgMain.SetButtons;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    lbEmptyDst: BOOLEAN;
    lbEmptySrc: BOOLEAN;
  BEGIN
    lbEmptySrc := lstSrc.Items.Count = 0;
    lbEmptyDst := lstDst.Items.Count = 0;
    btnInclude.Enabled := NOT lbEmptySrc;
    btnIncAll.Enabled  := NOT lbEmptySrc;
    btnExclude.Enabled := NOT lbEmptyDst;
    btnExAll.Enabled   := NOT lbEmptyDst;
    btnDelAll.Enabled  := NOT (lbEmptyDst AND lbEmptySrc);
{}END {SetButtons};




{}PROCEDURE   TdlgMain.SetItem(aoList: TListBox; aiIndex: INTEGER);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    liMaxIndex: INTEGER;
  BEGIN
    WITH aoList DO BEGIN
      SetFocus;
      liMaxIndex := aoList.Items.Count - 1;
      IF aiIndex = LB_ERR THEN BEGIN
        aiIndex := 0;
      END ELSE
      IF aiIndex > liMaxIndex THEN BEGIN
        aiIndex := liMaxIndex;
      END {IF};

      Selected[aiIndex] := TRUE;
    END {WITH};

    SetButtons;
{}END {SetItem};




{}PROCEDURE   TdlgMain.UpdateReuseIt;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    chkReuseIt.Enabled := goFileList.Count > 0;
{}END {UpdateReuseIt};


END {FrmMain}.
