UNIT KDStruct;
{+F
---------------------------------------------------------------------------
  File        - KDSTRUCT.PAS
  Purpose     - Provide common data structures.
  Author      - Keith S. Brown
  Copyright    1991..2000 by KSS (Keith S. Brown)
  Revised     - 2000.08.20(KSB) Based on KbaseObj (1991)
---------------------------------------------------------------------------}
INTERFACE
TYPE
  TTreeNode   = CLASS
  PUBLIC
    Left      : TTreeNode;
    Right     : TTreeNode;
  END {CLASS};

  TreeAction  = PROCEDURE(aoNode:TTreeNode);
  TreeCreate  = FUNCTION(avKey:VARIANT):TTreeNode;

  TTree       = CLASS
  PROTECTED
    Root      : TTreeNode;
  PUBLIC
    PROCEDURE   Clear;
    FUNCTION    Compare(avKey1,avKey2:VARIANT):INTEGER; VIRTUAL;
    CONSTRUCTOR Create;
    DESTRUCTOR  Destroy; OVERRIDE;
    FUNCTION    Empty : BOOLEAN;
    FUNCTION    Find(avKey:VARIANT):TTreeNode;
    PROCEDURE   ForEach(action:TreeAction);
    FUNCTION    GetKey(aoNode:TTreeNode):VARIANT; VIRTUAL;
    PROCEDURE   Insert(aoNode:TTreeNode);
    FUNCTION    Search(avKey:VARIANT; create:TreeCreate):TTreeNode;
  END {CLASS};

              {=================================================}
IMPLEMENTATION
USES
  SysUtils;


VAR
  uoNewTreeNode : TTreeNode;



{}FUNCTION    GetTreeNode(avKey:VARIANT):TTreeNode;
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    result := uoNewTreeNode;
{}END {GetTreeNodePtr};




{}PROCEDURE   DelTreeNode(aoNode:TTreeNode);
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    FreeAndNil(aoNode);
{}END {DelTreeNode};




{}PROCEDURE   TTree.Clear;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    ForEach(DelTreeNode);
    Root := NIL;
{}END {Clear};




{}FUNCTION    TTree.Compare(avKey1,avKey2:VARIANT):INTEGER;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    IF avKey1 < avKey2 THEN BEGIN
      result := -1;
    END ELSE
    IF avKey1 > avKey2 THEN BEGIN
      result := +1;
    END ELSE BEGIN
      result := 0;
    END {IF};
{}END {Compare};




{}CONSTRUCTOR TTree.Create;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    INHERITED Create;
    Root := NIL;
{}END {Create};




{}DESTRUCTOR  TTree.Destroy;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    Clear;
    INHERITED Destroy;
{}END {Destroy};




{}FUNCTION    TTree.Empty:BOOLEAN;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    Empty := Root = NIL;
{}END {Empty};




{}FUNCTION    TTree.Find(avKey:VARIANT):TTreeNode;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    uoNewTreeNode := NIL;
    Find := Search(avKey,GetTreeNode);
{}END {Find};




{}PROCEDURE   TTree.ForEach(action:TreeAction);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
{}{}PROCEDURE Traverse(p:TTreeNode);
    VAR
      r  : TTreeNode;
    BEGIN
      IF p <> NIL THEN BEGIN
        r := p.Right;
        Traverse(p.Left);
        Action(p);
        Traverse(r);
      END {IF};
{}{}END {Traverse};


  BEGIN
    Traverse(Root);
{}END {ForEach};




{}FUNCTION    TTree.GetKey(aoNode:TTreeNode):VARIANT;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    result := INTEGER(aoNode);
{}END {GetKey};




{}PROCEDURE   TTree.Insert(aoNode:TTreeNode);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    uoNewTreeNode := aoNode;
    aoNode := Search(GetKey(aoNode),GetTreeNode);
{}END {Insert};




{}FUNCTION    TTree.Search(avKey:VARIANT; Create:TreeCreate):TTreeNode;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
{}{}PROCEDURE Traverse(VAR aoNode:TTreeNode);
    VAR
      liComp  : INTEGER;
    BEGIN
      IF aoNode = NIL THEN BEGIN
        aoNode := Create(avKey);
        IF aoNode <> NIL THEN BEGIN        {don't access aoNode if not created}
          aoNode.Left  := NIL;            { ie., Create is a dummy for a}
          aoNode.Right := NIL;            { TTree.Find() call.           }
        END {IF};

        Search := aoNode;
      END ELSE BEGIN
        liComp := Compare(avKey,GetKey(aoNode));
        IF liComp < 0 THEN
          Traverse(aoNode.Left)
        ELSE
        IF liComp > 0 THEN
          Traverse(aoNode.Right)
        ELSE BEGIN
          Search := aoNode;
        END {IF};
      END {IF};
{}{}END {Traverse};


  BEGIN
    Traverse(Root);
{}END {Search};


END {KDStruct}.
