UNIT KFrmtGlbl;
{+F
---------------------------------------------------------------------------
  File        - KFRMTGLBL.PAS
  Purpose     -
  Author      - Keith S. Brown
  Copyright    2000 by KSS (Keith S. Brown) All rights reserved.
  Revised     - 2000.08.03(KSB) Wrote initial version.
---------------------------------------------------------------------------}
INTERFACE
CONST
  chPeriod    = '.';
  chQuote     = '''';
  chSpace     = ' ';


TYPE
{.F-}
                {....^....1....^., ....^....1....^., ....^....1....^., ....^....1....^., }
  TTokenKinds = (tkCommentBrace,   tkCommentPStar,   tkCommentSlash,   tkIdentifier,
                 tkLiteralFloat,   tkLiteralInteger, tkLiteralString,  tkSymbol,
                 tkFormattedBlock,
              {..Symbols..}
                 symAmpersand,     symAssign,        symAt,            symBrcktLeft,
                 symBrcktRight,    symCarrot,        symColon,         symComma,
                 symEqual,         symGE,            symGT,            symLE,
                 symLT,            symMinus,         symNE,            symParenLeft,
                 symParenRight,    symPlus,          symRange,         symStar,
                 symSemiColon,     symPeriod,        symSlash,
              {..Reserved Words..}
                 rwAnd,            rwArray,          rwAs,             rwAsm,
                 rwBegin,          rwCase,           rwClass,          rwConst,
                 rwConstructor,    rwDestructor,     rwDispInterface,  rwDiv,
                 rwDo,             rwDownTo,         rwElse,           rwEnd,
                 rwExcept,         rwExports,        rwFile,           rwFinalization,
                 rwFinally,        rwFor,            rwFunction,       rwGoto,
                 rwIf,             rwImplementation, rwIn,             rwInherited,
                 rwInitialization, rwInline,         rwInterface,      rwIs,
                 rwLabel,          rwLibrary,        rwMod,            rwNil,
                 rwNot,            rwObject,         rwOf,             rwOr,
                 rwOut,            rwPacked,         rwProcedure,      rwProgram,
                 rwProperty,       rwRaise,          rwRecord,         rwRepeat,
                 rwResourceString, rwSet,            rwShl,            rwShr,
                 rwString,         rwThen,           rwThreadVar,      rwTo,
                 rwTry,            rwType,           rwUnit,           rwUntil,
                 rwUses,           rwVar,            rwWhile,          rwWith,
                 rwXor,
              {..Directives..}
                 dwAbsolute,       dwAbstract,       dwAssembler,      dwAutomated,
                 dwCDecl,          dwContains,       dwDefault,        dwDispId,
                 dwDynamic,        dwExport,         dwExternal,       dwFar,
                 dwForward,        dwImplements,     dwIndex,          dwMessage,
                 dwName,           dwNear,           dwNoDefault,      dwOverload,
                 dwOverride,       dwPackage,        dwPascal,         dwPrivate,
                 dwProtected,      dwPublic,         dwPublished,      dwRead,
                 dwReadOnly,       dwRegister,       dwReintroduce,    dwRequires,
                 dwResident,       dwSafeCall,       dwStdCall,        dwStored,
                 dwVirtual,        dwWrite,          dwWriteOnly,
              {..Special Words..}
                 swAt,             swOn
                 );

  TKSSDirectives   = (
                 kdNone,           kdReformatOff,    kdReformatOn
                 );
{.F+}

{}FUNCTION    IdentifierKind(asIdentifier:STRING):TTokenKinds;
{}FUNCTION    KDirectiveKind(asDirective:STRING):TKSSDirectives;
{}FUNCTION    KDirectiveStr(aeDirective:TKSSDirectives):STRING;
{}FUNCTION    Symbols(aeKind:TTokenKinds):STRING;
{}FUNCTION    TokenIs(aeKind:TTokenKinds):STRING;
{}FUNCTION    Words(aeKind:TTokenKinds):STRING;
              {=================================================}
IMPLEMENTATION
USES
  SysUtils,
  KSString;



{}FUNCTION    IdentifierKind(asIdentifier:STRING):TTokenKinds;
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    le        : TTokenKinds;
  BEGIN
    result := tkIdentifier;
    asIdentifier := UpperCase(asIdentifier);
    FOR le := rwAnd TO swOn DO BEGIN
      IF asIdentifier = Words(le) THEN BEGIN
        result := le;
        Break;
      END {IF};
    END {FOR};
{}END {IdentifierKind};





{}FUNCTION    KDirectiveKind(asDirective:STRING):TKSSDirectives;
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    le        : TKSSDirectives;
  BEGIN
    result := kdNone;
    IF Length(asDirective) > 0 THEN BEGIN
      CASE asDirective[1] OF
        '{' : Delete(asDirective,1,1);
        '(' : Delete(asDirective,1,2);
        '/' : Delete(asDirective,1,2);
      END {CASE};

      asDirective := UpperCase(asDirective);
      FOR le := Low(TKSSDirectives) TO High(TKSSDirectives) DO BEGIN
        IF asDirective = KDirectiveStr(le) THEN BEGIN
          result := le;
          Break;
        END {IF};
      END {FOR};
    END {IF};
{}END {KDirectiveKind};




{}FUNCTION    KDirectiveStr(aeDirective:TKSSDirectives):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    CASE aeDirective OF
      kdNone       : result := '';
      kdReformatOff: result := '.F-';
      kdReformatOn : result := '.F+';
      ELSE           result := '';
    END {CASE};
{}END {KDirectiveStr};




{}FUNCTION    Symbols(aeKind:TTokenKinds):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    CASE aeKind OF
      symAmpersand : result := '&';    {basm override symbol}
      symAssign    : result := ':=';
      symAt        : result := '@';
      symBrcktLeft : result := '[';
      symBrcktRight: result := ']';
      symCarrot    : result := '^';    {usually, but not always a pointer dereference}
      symColon     : result := ':';
      symComma     : result := ',';
      symEqual     : result := '=';
      symGE        : result := '>=';
      symGT        : result := '>';
      symLE        : result := '<=';
      symLT        : result := '<';
      symMinus     : result := '-';
      symNE        : result := '<>';
      symParenLeft : result := '(';
      symParenRight: result := ')';
      symPlus      : result := '+';
      symRange     : result := '..';
      symStar      : result := '*';
      symSemiColon : result := ';';
      symPeriod    : result := '.';
      symSlash     : result := '/';
      ELSE           result := '';
    END {CASE};
{}END {Symbols};




{}FUNCTION    TokenIs(aeKind:TTokenKinds):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    IF aeKind IN [symAmpersand..symSlash] THEN BEGIN
      result := Symbols(aeKind);
    END ELSE
    IF aeKind IN [rwAnd..swOn] THEN BEGIN
      result := Words(aeKind);
    END ELSE BEGIN
      CASE aeKind OF
        tkCommentBrace  : result := '<{...}>';
        tkCommentPStar  : result := '<(*...*)>';
        tkCommentSlash  : result := '<//...>';
        tkFormattedBlock: result := '<formatted-block>';
        tkIdentifier    : result := '<identifier>';
        tkLiteralFloat  : result := '<float>';
        tkLiteralInteger: result := '<integer>';
        tkLiteralString : result := '<string>';
        tkSymbol        : result := '<symbol>';
      END {CASE};
    END {IF};
{}END {TokenIs};




{}FUNCTION    Words(aeKind:TTokenKinds):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    CASE aeKind OF
              {..reserved words..}
      rwAnd             : result := 'AND';
      rwArray           : result := 'ARRAY';
      rwAs              : result := 'AS';
      rwAsm             : result := 'ASM';
      rwBegin           : result := 'BEGIN';
      rwCase            : result := 'CASE';
      rwClass           : result := 'CLASS';
      rwConst           : result := 'CONST';
      rwConstructor     : result := 'CONSTRUCTOR';
      rwDestructor      : result := 'DESTRUCTOR';
      rwDispInterface   : result := 'DISPINTERFACE';
      rwDiv             : result := 'DIV';
      rwDo              : result := 'DO';
      rwDownTo          : result := 'DOWNTO';
      rwElse            : result := 'ELSE';
      rwEnd             : result := 'END';
      rwExcept          : result := 'EXCEPT';
      rwExports         : result := 'EXPORTS';
      rwFile            : result := 'FILE';
      rwFinalization    : result := 'FINALIZATION';
      rwFinally         : result := 'FINALLY';
      rwFor             : result := 'FOR';
      rwFunction        : result := 'FUNCTION';
      rwGoto            : result := 'GOTO';
      rwIf              : result := 'IF';
      rwImplementation  : result := 'IMPLEMENTATION';
      rwIn              : result := 'IN';
      rwInherited       : result := 'INHERITED';
      rwInitialization  : result := 'INITIALIZATION';
      rwInline          : result := 'INLINE';
      rwInterface       : result := 'INTERFACE';
      rwIs              : result := 'IS';
      rwLabel           : result := 'LABEL';
      rwLibrary         : result := 'LIBRARY';
      rwMod             : result := 'MOD';
      rwNil             : result := 'NIL';
      rwNot             : result := 'NOT';
      rwObject          : result := 'OBJECT';
      rwOf              : result := 'OF';
      rwOr              : result := 'OR';
      rwOut             : result := 'OUT';
      rwPacked          : result := 'PACKED';
      rwProcedure       : result := 'PROCEDURE';
      rwProgram         : result := 'PROGRAM';
      rwProperty        : result := 'PROPERTY';
      rwRaise           : result := 'RAISE';
      rwRecord          : result := 'RECORD';
      rwRepeat          : result := 'REPEAT';
      rwResourceString  : result := 'RESOURCESTRING';
      rwSet             : result := 'SET';
      rwShl             : result := 'SHL';
      rwShr             : result := 'SHR';
      rwString          : result := 'STRING';
      rwThen            : result := 'THEN';
      rwThreadVar       : result := 'THREADVAR';
      rwTo              : result := 'TO';
      rwTry             : result := 'TRY';
      rwType            : result := 'TYPE';
      rwUnit            : result := 'UNIT';
      rwUntil           : result := 'UNTIL';
      rwUses            : result := 'USES';
      rwVar             : result := 'VAR';
      rwWhile           : result := 'WHILE';
      rwWith            : result := 'WITH';
      rwXor             : result := 'XOR';
              {..directive words..}
      dwAbsolute        : result := 'ABSOLUTE';
      dwAbstract        : result := 'ABSTRACT';
      dwAssembler       : result := 'ASSEMBLER';
      dwAutomated       : result := 'AUTOMATED';
      dwCDecl           : result := 'CDECL';
      dwContains        : result := 'CONTAINS';
      dwDefault         : result := 'DEFAULT';
      dwDispId          : result := 'DISPID';
      dwDynamic         : result := 'DYNAMIC';
//    dwExport          : result := 'EXPORT';
      dwExternal        : result := 'EXTERNAL';
      dwFar             : result := 'FAR';
      dwForward         : result := 'FORWARD';
      dwImplements      : result := 'IMPLEMENTS';
      dwIndex           : result := 'INDEX';
      dwMessage         : result := 'MESSAGE';
      dwName            : result := 'NAME';
      dwNear            : result := 'NEAR';
      dwNoDefault       : result := 'NODEFAULT';
      dwOverload        : result := 'OVERLOAD';
      dwOverride        : result := 'OVERRIDE';
      dwPackage         : result := 'PACKAGE';
      dwPascal          : result := 'PASCAL';
      dwPrivate         : result := 'PRIVATE';
      dwProtected       : result := 'PROTECTED';
      dwPublic          : result := 'PUBLIC';
      dwPublished       : result := 'PUBLISHED';
      dwRead            : result := 'READ';
      dwReadOnly        : result := 'READONLY';
      dwRegister        : result := 'REGISTER';
      dwReintroduce     : result := 'REINTRODUCE';
      dwRequires        : result := 'REQUIRES';
      dwResident        : result := 'RESIDENT';
      dwSafeCall        : result := 'SAFECALL';
      dwStdCall         : result := 'STDCALL';
      dwStored          : result := 'STORED';
      dwVirtual         : result := 'VIRTUAL';
      dwWrite           : result := 'WRITE';
      dwWriteOnly       : result := 'WRITEONLY';
              {..special words..}
      swAt              : result := 'AT';
      swOn              : result := 'ON';
      ELSE                result := '';
    END {CASE};
{}END {Words};




END {KFrmtGlbl}.
