UNIT KSSAbout;
{+H
---------------------------------------------------------------------------
  File        - KSSABOUT.PAS
  Purpose     - Generalized About box.
  Author      - Keith S. Brown
  Copyright    2000 by KSS. All rights reserved.
  Revised     - 2000.0312 (KSB) Developed initial version.
              - 2000.0727 (KSB) Fixed credit logic.
---------------------------------------------------------------------------}
INTERFACE
USES
  Windows,  SysUtils, Classes,  Graphics, Forms,    Controls, StdCtrls,
  Buttons,  ExtCtrls, URLLabel;

TYPE
  TAboutBox   = CLASS(TForm)
    btnAssist      : TBitBtn;
    btnOk          : TBitBtn;
    iconProg1      : TImage;
    iconProg2      : TImage;
    imgDelphi      : TImage;
    lblCopyright   : TLabel;
    lblComments    : TLabel;
    lblDelphi      : TURLLabel;
    lblProductName : TLabel;
    lblRef1        : TLabel;
    lblRef2        : TLabel;
    lblURL         : TURLLabel;
    lblVersion     : TLabel;
    pnlAbout       : TPanel;
    pnlInfo        : TPanel;
    pnlRef         : TPanel;
    PROCEDURE   DoShowAssist(aoSender: TObject);
    PROCEDURE   FormClose(aoSender: TObject; VAR Action: TCloseAction);
    PROCEDURE   FormCreate(aoSender: TObject);
    PROCEDURE   FormShow(aoSender: TObject);
  PRIVATE
    FiBase         : INTEGER;
    FiIndex        : INTEGER;
    FiYear         : INTEGER;
    FsApplication  : STRING;
    FsReference    : STRING;
    FsVersion      : STRING;
  PROTECTED
    FoVersioning   : TStringList;
    FUNCTION    GetVersionInfo:INTEGER;
    PROCEDURE   SetStr(asValue:STRING);
  PUBLIC
    PROPERTY    CopyRightYear:INTEGER WRITE FiYear;
    PROPERTY    ProgramName  :STRING  WRITE FsApplication;
    PROPERTY    Reference    :STRING  WRITE FsReference;
    PROPERTY    Version      :STRING  READ  FsVersion WRITE FsVersion;
    PROPERTY    WhatsOnURL   :STRING  WRITE SetStr;
  END {CLASS};

              {=================================================}
IMPLEMENTATION
{$R *.DFM}

CONST
  chNewLine        = ^M^J;
  ciCompanyName    = 0;
  ciFileDescription= 1;
  ciFileVersion    = 2; {* FsVersion}
  ciInternalName   = 3;
  ciLegalCopyright = 4; {* FiYear}
  ciLegalTrademarks= 5;
  ciOriginlFileName= 6;
  ciProductName    = 7; {* FsApplication}
  ciProductVersion = 8;
  ciComments       = 9; {* FsReference}


{}PROCEDURE   TAboutBox.DoShowAssist(aoSender: TObject);
{+O
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    li        : INTEGER;
  BEGIN
    lblComments.Caption := '';
    FOR li := 1 TO (FoVersioning.Count-FiBase-1) DO BEGIN
      IF FiIndex >= (FoVersioning.Count-FiBase) THEN BEGIN
        FiIndex := 0;
      END {IF};

      IF lblComments.Caption <> '' THEN BEGIN
        lblComments.Caption := lblComments.Caption + chNewLine;
      END {IF};

      lblComments.Caption := FoVersioning[FiIndex+FiBase]+chNewLine;
      Inc(FiIndex);
    END {IF};
{}END {DoShowAssist};


{}PROCEDURE   TAboutBox.FormClose(aoSender: TObject; VAR Action: TCloseAction);
{+O
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    FoVersioning.Free;
    action := caFree;
{}END {FormClose};




{}PROCEDURE   TAboutBox.FormCreate(aoSender: TObject);
{+O
---------------------------------------------------------------------------
  Purpose     - List Credits.
---------------------------------------------------------------------------}
  BEGIN
    FoVersioning := TStringList.Create;
    FiBase := GetVersionInfo;
    FiYear := 0;
    FiIndex:= 1;
    FoVersioning.Add('Borland.com - Delphi');
    FoVersioning.Add('JG Soft - URL Label');
    lblComments.Caption := FoVersioning[FiBase];
{}END {FormCreate};




{}PROCEDURE   TAboutBox.FormShow(aoSender: TObject);
{+O
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    lsYear    : STRING;
  BEGIN
    lsYear := IntToStr(FiYear);
    IF FiBase >= 10 THEN BEGIN
      IF (FsApplication = '') THEN BEGIN
        FsApplication := FoVersioning[ciProductName];
      END {IF};

      IF (FsVersion = '') THEN BEGIN
        FsVersion := FoVersioning[ciFileVersion];
      END {IF};

      IF (FsReference = '') THEN BEGIN
        FsReference := FoVersioning[ciComments];
      END {IF};

      IF (FiYear = 0) THEN BEGIN
        lsYear := FoVersioning[ciLegalCopyright];
      END {IF};
    END {IF};

    self.caption := 'About '+FsApplication;
    lblVersion.caption := 'Version '+FsVersion;
    lblProductName.caption := FsApplication;
    lblCopyright.caption := 'Copyright (c) '+lsYear+' KSS. All rights reserved.';
    IF Length(FsReference) > 0 THEN BEGIN
      FoVersioning.Add(FsReference);
//    lblComments.caption := lblComments.caption + FsReference;
    END {IF};
{}END {FormShow};




{}FUNCTION    TAboutBox.GetVersionInfo:INTEGER;
{+O
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  CONST
    ciFields  = 10;
    caFldNames: ARRAY[1..ciFields]OF STRING =
              ('CompanyName',   'FileDescription','FileVersion',     'InternalName',
               'LegalCopyright','LegalTradeMarks','OriginalFileName','ProductName',
               'ProductVersion','Comments');
  VAR
    li        : INTEGER;
    liLen     : UINT;
    liSize    : DWORD;
    lp        : PChar;
    lpValue   : PChar;
    ls        : STRING;
  BEGIN
    result := 0;
    ls := application.ExeName;
    liSize := GetFileVersionInfoSize(PChar(ls),liSize);
    IF liSize > 0 THEN BEGIN
      lp := AllocMem(liSize);
      GetFileVersionInfo(PChar(ls),0,liSize,lp);
      FOR li := 1 TO ciFields DO BEGIN
        IF VerQueryValue(lp,PChar('StringFileInfo\040904E4\'+caFldNames[li]),POINTER(lpValue),liLen) THEN BEGIN
          FoVersioning.Add(lpValue);
          Inc(result);
        END {IF};
      END {FOR};

      FreeMem(lp,liSize);
    END {IF};
{}END {GetVersionInfo};




{}PROCEDURE   TAboutBox.SetStr(asValue:STRING);
{+O
---------------------------------------------------------------------------
  Purpose     - 
---------------------------------------------------------------------------}
  BEGIN
    IF asValue <> '' THEN BEGIN
      lblRef2.Caption := asValue;
    END {IF};
{}END {SetStr};




END {DlgAbout}.
