UNIT KSSbuffer;
{+F
---------------------------------------------------------------------------
  File        - KSSBUFFER.PAS
  Purpose     -
  Author      - Keith S. Brown
  Copyright    2000 by KSS (Keith S. Brown) All rights reserved.
  Revised     - 2000.0728 (KSB) Wrote initial version.
              - 2000.0729 (KSB) Added LineNo property. Fixed internal logic.
              - 2000.0730 (KSB) Given that I'm not changing the string list
                that contains the lines I'm parsing, there is no need to
                copy it by doing an Assign. Changed the code accordingly.
              - 2000.08.03(KSB) Added ColumnNo property.
              - 2000.08.21(KSB) Added Clear method.
---------------------------------------------------------------------------}
INTERFACE
USES
  Classes;


CONST
  chNewLine   = ^M;
TYPE
  TKSSstrBfr   = CLASS(TObject)
  PRIVATE
    FbEmpty   : BOOLEAN;
    FiColNo   : INTEGER;
    FiLineNo  : INTEGER;
    FoLines   : TStrings;
    FsBuffer  : STRING;
  PROTECTED
    PROCEDURE   SetLines(aoLines:TStrings);
    PROCEDURE   UpdateInternalBuffer;
  PUBLIC
    PROPERTY    ColumnNo:INTEGER  READ FiColNo;
    PROPERTY    IsEmpty :BOOLEAN  READ FbEmpty;
    PROPERTY    LineNo  :INTEGER  READ FiLineNo;
    PROPERTY    Lines   :TStrings READ FoLines WRITE SetLines;
  PUBLIC
//  PROCEDURE   Assign(aoSource:TPersistent); OVERRIDE;
    PROCEDURE   Clear;
    CONSTRUCTOR Create;
    FUNCTION    GetChar:CHAR;
    FUNCTION    NextCharIs:CHAR;
    PROCEDURE   PushBack(asExtra:STRING);
  END {CLASS};

              {=================================================}
IMPLEMENTATION

(*
{}PROCEDURE   TKSSstrBfr.Assign(aoSource:TPersistent);
{+O
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    FbEmpty := FALSE;
    FiLineNo := 0;
    FsBuffer:= '';
    INHERITED Assign(aoSource);
{}END {Assign};
*)



{}PROCEDURE   TKSSstrBfr.Clear;
{+O
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    FbEmpty := TRUE;
    FiLineNo:= 0;
    FiColNo := 0;
    FsBuffer:= '';
    IF Assigned(FoLines) THEN BEGIN
      FoLines.Clear;
    END {IF};
{}END {Clear};




{}CONSTRUCTOR TKSSstrBfr.Create;
{+O
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    INHERITED Create;
    FiLineNo := 0;
    FbEmpty := TRUE;
{}END {Create};




{}FUNCTION    TKSSstrBfr.GetChar:CHAR;
{+O
---------------------------------------------------------------------------
  Purpose     - Returns the next available character.
  Remarks     - The returned character is deleted from the buffer.
---------------------------------------------------------------------------}
  BEGIN
    UpdateInternalBuffer;
    IF Length(FsBuffer) > 0 THEN BEGIN
      result := FsBuffer[1];
      System.Delete(FsBuffer,1,1);
      Inc(FiColNo);
    END ELSE BEGIN
      result := #0;
      FbEmpty:= TRUE;
      FiColNo:= 0;
    END {IF};
{}END {GetChar};




{}FUNCTION    TKSSstrBfr.NextCharIs:CHAR;
{+O
---------------------------------------------------------------------------
  Purpose     - Returns the next available character.
---------------------------------------------------------------------------}
  BEGIN
    UpdateInternalBuffer;
    IF Length(FsBuffer) > 0 THEN BEGIN
      result := FsBuffer[1];
    END ELSE BEGIN
      result := #0;
      FbEmpty:= TRUE;
    END {IF};
{}END {NextCharIs};




{}PROCEDURE   TKSSstrBfr.PushBack(asExtra:STRING);
{+O
---------------------------------------------------------------------------
  Purpose     - Pushes a character string to the front.
---------------------------------------------------------------------------}
  BEGIN
    FsBuffer := asExtra + FsBuffer;
{}END {PushBack};




{}PROCEDURE   TKSSstrBfr.SetLines(aoLines:TStrings);
{+O
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    FoLines := aoLines;
    FbEmpty := FoLines.Count = 0;
{}END {SetLines};




{}PROCEDURE   TKSSstrBfr.UpdateInternalBuffer;
{+O
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    IF Length(FsBuffer) = 0 THEN BEGIN
      Inc(FiLineNo);
      FiColNo := 0;
      IF Assigned(FoLines) AND (FoLines.Count > 0) AND ((FiLineNo-1) < FoLines.Count) THEN BEGIN
        FsBuffer := FoLines.Strings[FiLineNo-1]+chNewLine;
      END ELSE BEGIN
        FbEmpty:= TRUE;
      END {IF};
    END {IF};
{}END {UpdateInternalBuffer};


END {KSSbuffer}.
