UNIT KSSscaner;
{+F
---------------------------------------------------------------------------
  File        - KSSSCANER.PAS
  Purpose     - Scan Pascal text.
  Author      - Keith S. Brown
  Copyright    2000 by KSS (Keith S. Brown) All rights reserved.
  Revised     - 2000.08.03 (KSB) Wrote initial version.
---------------------------------------------------------------------------}
INTERFACE
USES
  Classes,  KSSbuffer,KFrmtGlbl;


TYPE
  TFormatOffEvent  = PROCEDURE (aoSender:TObject; VAR asValue:STRING; VAR aeKind:TTokenKinds)OF OBJECT;
  TKSSPasScanner   = CLASS
  PRIVATE
    FoSrcBfr       : TKSSstrBfr;
    FOnFormatOff   : TFormatOffEvent;
    FOnFormatOn    : TNotifyEvent;
  PROTECTED
    PROPERTY    OnFormatOff : TFormatOffEvent READ FOnFormatOff WRITE FOnFormatOff;
    PROPERTY    OnFormatOn  : TNotifyEvent    READ FOnFOrmatOn  WRITE FOnFormatOn;
    PROPERTY    SrcBuffer   : TKSSstrBfr      READ FoSrcBfr     WRITE FoSrcBfr;
  PROTECTED
    FUNCTION    GetCommentBrace(VAR aeKind:TTokenKinds):STRING; VIRTUAL;
    FUNCTION    GetCommentPStar(VAR aeKind:TTokenKinds):STRING; VIRTUAL;
    FUNCTION    GetCommentSlash(VAR aeKind:TTokenKinds):STRING; VIRTUAL;
    FUNCTION    GetHexadecimal:STRING;  VIRTUAL;
    FUNCTION    GetIdentifier(VAR aeKind:TTokenKinds):STRING;  VIRTUAL;
    FUNCTION    GetIsEmpty:BOOLEAN; VIRTUAL;
    FUNCTION    GetNumber(VAR aeKind:TTokenKinds):STRING;  VIRTUAL;
    FUNCTION    GetPreformattedBlock:STRING;
    FUNCTION    GetString(VAR aeKind:TTokenKinds):STRING;  VIRTUAL;
    FUNCTION    GetSource:TStrings;
    PROCEDURE   SetSource(aoList:TStrings);
    FUNCTION    WhatKind(VAR asImage:STRING):TTokenKinds;  VIRTUAL;
  PUBLIC
    PROPERTY    IsEmpty:BOOLEAN READ GetIsEmpty;
    PROPERTY    Source:TStrings READ GetSource WRITE SetSource;
  PUBLIC
    CONSTRUCTOR Create;
    DESTRUCTOR  Destroy; OVERRIDE;
    FUNCTION    ScanCode:BOOLEAN; VIRTUAL;
  END {CLASS};


              {=================================================}
IMPLEMENTATION
USES
  SysUtils,
  KSString;

  
{}CONSTRUCTOR TKSSPasScanner.Create;
{+M
---------------------------------------------------------------------------
  Purpose     - Initialize the Pascal Parser.
---------------------------------------------------------------------------}
  BEGIN
    FoSrcBfr:= TKSSstrBfr.Create;
{}END {Create};




{}DESTRUCTOR  TKSSPasScanner.Destroy;
{+M
---------------------------------------------------------------------------
  Purpose     - Release allocated memory and destroy the Parser.
---------------------------------------------------------------------------}
  BEGIN
    FoSrcBfr.Free;
    INHERITED Destroy;
{}END {Destroy};




{}FUNCTION    TKSSPasScanner.GetCommentBrace(VAR aeKind:TTokenKinds):STRING;
{+M
---------------------------------------------------------------------------
  Purpose     - Get a comment that starts with a brace "{"
---------------------------------------------------------------------------}
  VAR
    le        : TKSSDirectives;
  BEGIN
    aeKind := tkCommentBrace;
    result := SrcBuffer.GetChar;
    REPEAT
      result := result + SrcBuffer.GetChar;
    UNTIL result[Length(result)] = '}';

    IF Length(result) = 2 THEN BEGIN
      result := '';
    END {IF};

    IF Length(result) > 4 THEN BEGIN
      le := KDirectiveKind(Copy(result,1,4));
      CASE le OF
        kdReformatOff : BEGIN
              {1. copy code to result until .F+ directive is found}
          IF Assigned(FOnFormatOff) THEN BEGIN
            FOnFormatOff(self,result,aeKind);
          END {IF};
        END {kdReformatOff};

        kdReformatOn : BEGIN
              {2. find the previously stored formatted block and set the "jump" value}
          IF Assigned(FOnFormatOn) THEN BEGIN
            FOnFormatOn(self);
          END {IF};
        END {kdReformatOn};
      END {CASE};
    END {IF};
{}END {GetCommentBrace};




{}FUNCTION    TKSSPasScanner.GetCommentPStar(VAR aeKind:TTokenKinds):STRING;
{+M
---------------------------------------------------------------------------
  Purpose     - Get a comment that starts with a "(*"
---------------------------------------------------------------------------}
  BEGIN
    aeKind := tkCommentPStar;
    result := SrcBuffer.GetChar + SrcBuffer.GetChar;
    REPEAT
      result := result + SrcBuffer.GetChar;
    UNTIL (result[Length(result)] = '*') AND (SrcBuffer.NextCharIs = ')');

    result := result + SrcBuffer.GetChar;
{}END {GetCommentPStar};




{}FUNCTION    TKSSPasScanner.GetCommentSlash(VAR aeKind:TTokenKinds):STRING;
{+M
---------------------------------------------------------------------------
  Purpose     - Get a comment that starts with a "//"
---------------------------------------------------------------------------}
  BEGIN
    aeKind := tkCommentSlash;
    result := SrcBuffer.GetChar + SrcBuffer.GetChar;
    WHILE NOT (SrcBuffer.NextCharIs IN [^M,^J]) DO BEGIN
      result := result + SrcBuffer.GetChar;
    END {WHILE};
{}END {GetCommentSlash};




{}FUNCTION    TKSSPasScanner.GetHexadecimal:STRING;
{+M
---------------------------------------------------------------------------
  Purpose     - Get a hexadecimal number, for example "$ABC9"
---------------------------------------------------------------------------}
  BEGIN
    result := SrcBuffer.GetChar;
    WHILE UpCase(SrcBuffer.NextCharIs) IN ['A'..'F','0'..'9'] DO BEGIN
      result := result + SrcBuffer.GetChar;
    END {WHILE};
{}END {GetHexadecimal};




{}FUNCTION    TKSSPasScanner.GetIdentifier(VAR aeKind:TTokenKinds):STRING;
{+M
---------------------------------------------------------------------------
  Purpose     - Get an identifier (which may be a reserved word, a directive
                word, or a special word).
---------------------------------------------------------------------------}
  BEGIN
    result := SrcBuffer.GetChar;
    WHILE UpCase(SrcBuffer.NextCharIs) IN ['_','A'..'Z','0'..'9'] DO BEGIN
      result := result + SrcBuffer.GetChar;
    END {WHILE};

    aeKind := IdentifierKind(result);
{}END {GetIdentifier};




{}FUNCTION    TKSSPasScanner.GetIsEmpty:BOOLEAN;
{+M
---------------------------------------------------------------------------
  Purpose     - Property "Getter"
---------------------------------------------------------------------------}
  BEGIN
    result := SrcBuffer.IsEmpty;
{}END {GetIsEmpty};




{}FUNCTION    TKSSPasScanner.GetNumber(VAR aeKind:TTokenKinds):STRING;
{+M
---------------------------------------------------------------------------
  Purpose     - Get a number (which could be either an integer or a float)
---------------------------------------------------------------------------}
{}{}PROCEDURE AppendDigit;
    BEGIN
      WHILE SrcBuffer.NextCharIs IN ['0'..'9'] DO BEGIN
        result := result + SrcBuffer.GetChar;
      END {WHILE};
{}{}END {AppendDigit};


  BEGIN
    aeKind := tkLiteralInteger;
    result := SrcBuffer.GetChar;
    AppendDigit;

    IF SrcBuffer.NextCharIs IN [chPeriod, 'E'] THEN BEGIN
      IF SrcBuffer.NextCharIs = chPeriod THEN BEGIN
        SrcBuffer.GetChar;
        IF SrcBuffer.NextCharIs = chPeriod THEN BEGIN
          SrcBuffer.PushBack(chPeriod);
        END ELSE BEGIN
          result := result + chPeriod;
          AppendDigit;
        END {IF};
      END {IF};

      IF UpCase(SrcBuffer.NextCharIs) = 'E' THEN BEGIN
        SrcBuffer.GetChar;
        result := result + 'E';
        IF SrcBuffer.NextCharIs IN ['+','-'] THEN BEGIN
          result := result + SrcBuffer.GetChar;
        END {IF};

        AppendDigit;
      END {IF};

      aeKind := tkLiteralFloat;
    END {IF};
{}END {GetNumber};




{}FUNCTION    TKSSPasScanner.GetPreformattedBlock:STRING;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  LABEL         LRestart;
  VAR
    leComment : TTokenKinds;
    ls        : STRING;
  BEGIN
    result := SrcBuffer.GetChar;
LRestart:
    REPEAT
      result := result + SrcBuffer.GetChar;
      IF SrcBuffer.IsEmpty THEN BEGIN
        Exit;
      END {IF};

      IF (LastChars(result,1) = '{') THEN BEGIN
        leComment := tkCommentBrace;
      END ELSE
      IF (LastChars(result,2) = '(*') THEN BEGIN
        leComment := tkCommentPStar;
      END ELSE
      IF (LastChars(result,2) = '//') THEN BEGIN
        leComment := tkCommentSlash;
      END ELSE BEGIN
        leComment := tkFormattedBlock;
      END {IF};
    UNTIL (leComment <> tkFormattedBlock);

    ls := SrcBuffer.GetChar + SrcBuffer.GetChar + SrcBuffer.GetChar;
    result := result + ls;
    IF KDirectiveKind(ls) = kdReformatOn THEN BEGIN
      CASE leComment OF
        tkCommentBrace : BEGIN
          REPEAT
            result := result + SrcBuffer.GetChar;
          UNTIL LastChars(result,1) = '}';
        END {tkCommentBrace};

        tkCommentPStar : BEGIN
          REPEAT
            result := result + SrcBuffer.GetChar;
          UNTIL LastChars(result,2) = '*)';
        END {tkCommentPStar};

        tkCommentSlash : BEGIN
          WHILE SrcBuffer.NextCharIs <> chNewLine DO BEGIN
            result := result + SrcBuffer.GetChar;
          END {WHILE};
        END {tkCommentSlash}
      END {CASE};
    END ELSE BEGIN
      GOTO LRestart;
    END {IF};
{}END {GetPreformattedBlock};




{}FUNCTION    TKSSPasScanner.GetSource:TStrings;
{+M
---------------------------------------------------------------------------
  Purpose     - Get method for the Source property.
---------------------------------------------------------------------------}
  BEGIN
    result := SrcBuffer.Lines;
{}END {GetSource};




{}FUNCTION    TKSSPasScanner.GetString(VAR aeKind:TTokenKinds):STRING;
{+M
---------------------------------------------------------------------------
  Purpose     - Get a string.
  Remarks     - A pascal string literal is:
                1. an empty string, ie., two adjacent quotes: ''
                2. a set of characters enclosed by quotes, ie., 'Hello'
                3. a quoted quote, ie., '''' which is a string containing a single quote.
                4. a control string which is a sequence of one or more control characters,
                   each of which consists of the # symbol followed by an unsigned
                   integer constant from 0 to 255 (decimal or hexadecimal) that denotes
                   the corresponding ASCII character.
                5. an "old" control character is a "^" immediately followed by a single
                   character. This is a holdover from old Pascal. If context is not
                   maintained it can be misinterpreted. For example: a^[1] should
                   be interpreted as "a" is a pointer to the array for which we want
                   element number 1. If context is not maintained, "a" will be correctly
                   identified as an identifier and ^[ will be interpreted as an old
                   control character (string) followed by the integer 1 and "]" --
                   obviously not what we want. Looking ahead does not help because
                   ^[ and ^. are both interpreted as characters (try it!).

                   Therefore context must be maintained by looking back. If "^" follows
                   an identified string, then it is obviously a part of that string ie.,
                   'How are you'^M^J
                   This function handles the above case.

                   However, if "^" follows a non-reserved word identifier, it can
                   be assumed to be the pointer "dereference operator" instead.
                   If the "^" follows a ")" (ie., dereferencing a typecast pointer)
                   or a "]" (deferencing an element of an array of pointers) the
                   "^" can be assumed to be the pointer "dereference operator."

                   On the other hand, if the "^" follows any other symbol other than
                   a ")" or a "]" it can be assumed that the "^" should be combined
                   with the next character to form a control character.

                   For example:
                       type
                         pr=^r;
                         r=record
                           a:integer;
                         end;
                       var
                         m   : pr;
                         s   : string;
                       begin
                         New(m);
                         m ^ . a := 1;
                         s := ^a + ^b;
                       end;

                   Of course all this angst vanishes if a proper symbol table is
                   maintained and lookups performed.

  WARNING     - Note that 'Hello'#$0D^J'How are you!' puts "Hello" on one line
                and "How are you!" on the next. On the other hand, 'Hello''How are you!'
                simply inserts an apostrophe between "Hello" and "How" yielding
                "Hello'How are you!"
---------------------------------------------------------------------------}
{}{}PROCEDURE AppendChar;
    BEGIN           
      REPEAT
        result := result + SrcBuffer.GetChar;
      UNTIL result[Length(result)] = chQuote;
{}{}END {AppendChar};


  LABEL         LSubStr;
  VAR
    le        : TTokenKinds;
  BEGIN
    aeKind := tkLiteralString;
LSubStr:
    CASE SrcBuffer.NextCharIs OF
      '^' : BEGIN
        result := result + SrcBuffer.GetChar + SrcBuffer.GetChar;
        GOTO LSubStr;
      END {'^'};

      '#' : BEGIN
        result := result + SrcBuffer.GetChar;
        CASE SrcBuffer.NextCharIs OF
          '$' : result := result + GetHexadecimal;
          ELSE  result := result + GetNumber(le);
        END {CASE};

        GOTO LSubStr;
      END {'#'};

      chQuote : BEGIN
        result := SrcBuffer.GetChar;
        AppendChar;
        GOTO LSubStr;
      END {chQuote};
    END {CASE};
{}END {GetString};




{}FUNCTION    TKSSPasScanner.ScanCode:BOOLEAN;
{+M
---------------------------------------------------------------------------
  Purpose     - ScanCode a pascal source text
---------------------------------------------------------------------------}
  VAR
    le        : TTokenKinds;
    ls        : STRING;
  BEGIN
    result := FALSE;
    WHILE NOT SrcBuffer.IsEmpty DO BEGIN
      le := WhatKind(ls);

      CASE le OF
        tkCommentBrace,
        tkCommentPStar : BEGIN
          WHILE Length(ls) > 0 DO BEGIN
//          Output.Add(KSString.Parse(ls,chNewLine))
          END {WHILE};
        END {tkCommentBrace,tkCommentPStar};

//      ELSE Output.Add(ls);
      END {CASE};
    END {WHILE};
{}END {ScanCode};




{}PROCEDURE   TKSSPasScanner.SetSource(aoList:TStrings);
{+M
---------------------------------------------------------------------------
  Purpose     - Set method for the Source property.
  Remarks     - Set the lines property in the source buffer to TStrings to
                be parsed.
---------------------------------------------------------------------------}
  BEGIN
    SrcBuffer.Lines := aoList;
{}END {SetSource};




{}FUNCTION    TKSSPasScanner.WhatKind(VAR asImage:STRING):TTokenKinds;
{+M
---------------------------------------------------------------------------
  Purpose     - Return the next token and its "kind."
  Revised     - 2000.08.21(KSB) Added code for '$'.
---------------------------------------------------------------------------}
{}{}FUNCTION  GetOtherSymbol(VAR aeKind:TTokenKinds):STRING;
    VAR
      le      : TTokenKinds;
    BEGIN
      result := SrcBuffer.GetChar;
      FOR le := symAmpersand TO symSlash DO BEGIN
        IF Symbols(le) = result THEN BEGIN
          aeKind := le;
          Break;
        END {IF};
      END {FOR};
{}{}END {GetOtherSymbol};


  LABEL         LRetry;
  VAR
    ls        : STRING;
  BEGIN
LRetry:
    ls := '';
    CASE UpCase(SrcBuffer.NextCharIs) OF
      'A'..'Z','_' : ls := GetIdentifier(result);
      '0'..'9'     : ls := GetNumber(result);
      '#',chQuote  : ls := GetString(result);
      '$'          : ls := GetHexadecimal;

      '(' : BEGIN
        ls := SrcBuffer.GetChar;
        result := symParenLeft;
        CASE SrcBuffer.NextCharIs OF
          '*' : BEGIN
            SrcBuffer.PushBack(ls);
            ls := GetCommentPStar(result);
          END {'*'};

          '.' : BEGIN
            SrcBuffer.GetChar;
            ls := '[';
            result := symBrcktLeft;
          END {'.'};
        END {CASE};
      END {'('};

      '.' : BEGIN
        result := symPeriod;
        ls := SrcBuffer.GetChar;
        CASE SrcBuffer.NextCharIs OF
          ')' : BEGIN
            SrcBuffer.GetChar;
            ls := ']';
            result := symBrcktRight;
          END {')'};

          chPeriod : BEGIN
            ls := ls + SrcBuffer.GetChar;
            result := symRange;
          END {chPeriod};
        END {IF};
      END {'.'};

      '/' : BEGIN
        ls := SrcBuffer.GetChar;
        CASE SrcBuffer.NextCharIs OF
          '/' : BEGIN
            SrcBuffer.PushBack(ls);
            ls := GetCommentSlash(result);
          END {'/'};

          ELSE  result := symSlash;
        END {CASE};
      END {'/'};

      ':' : BEGIN
        result := symColon;
        ls := SrcBuffer.GetChar;
        CASE SrcBuffer.NextCharIs OF
          '=' : BEGIN
            ls := ls + SrcBuffer.GetChar;
            result := symAssign;
          END {'='};
        END {CASE};
      END {':'};

      '<' : BEGIN
        result := symLT;
        ls := SrcBuffer.GetChar;
        CASE SrcBuffer.NextCharIs OF
          '=' : BEGIN
            ls := ls + SrcBuffer.GetChar;
            result := symLE;
          END {'='};

          '>' : BEGIN
            ls := ls + SrcBuffer.GetChar;
            result := symNE;
          END {'>'};
        END {CASE};
      END {'<'};

      '>' : BEGIN
        result := symGT;
        ls := SrcBuffer.GetChar;
        CASE SrcBuffer.NextCharIs OF
          '=' : BEGIN
            ls := ls + SrcBuffer.GetChar;
            result := symGE;
          END {'='};
        END {CASE};
      END {'>'};

      '{' : ls := GetCommentBrace(result);

      '&', {basm override symbol}
      '^', ')', '*', '+', ',', '-', ';', '=', '@', '[',
      ']' : BEGIN
        ls := GetOtherSymbol(result);
      END {'&', '^', ')', '*', '+', ',', '-', ';', '=', '@', '[', ']'};

      ELSE BEGIN
        SrcBuffer.GetChar;
        IF NOT SrcBuffer.IsEmpty THEN BEGIN
          GOTO LRetry;
        END {IF};
      END {ELSE};
    END {CASE};

    asImage := ls;
{}END {WhatKind};




END {KSSscanner}.
