UNIT KSSsymbols;
{+F
---------------------------------------------------------------------------
  File        - KSSSYMBOLS.PAS
  Purpose     - Defines scanner nodes.
  Author      - Keith S. Brown
  Copyright    2000 by KSS (Keith S. Brown) All rights reserved.
  Revised     -
---------------------------------------------------------------------------}
INTERFACE
USES
  Classes,
  KFrmtGlbl;

TYPE
  TListNode   = CLASS;
  TBaseNode   = CLASS(TObject)
  PRIVATE
    FiNext    : LongINT;
    FoCBlock  : TListNode;
  PROTECTED
    FUNCTION    GetHasBlock:BOOLEAN; VIRTUAL;
    FUNCTION    GetIsWord : BOOLEAN; VIRTUAL; ABSTRACT;
    FUNCTION    GetStr:STRING; VIRTUAL; ABSTRACT;
  PUBLIC
    PROPERTY    CBlock  : TListNode READ FoCBlock WRITE FoCBlock;
    PROPERTY    HasBlock: BOOLEAN   READ GetHasBlock;
    PROPERTY    Image   : STRING    READ GetStr;
    PROPERTY    IsWord  : BOOLEAN   READ GetIsWord;
    PROPERTY    Next    : LongINT   READ FiNext   WRITE FiNext;
  PUBLIC
    CONSTRUCTOR Create;
    DESTRUCTOR  Destroy; OVERRIDE;
  END {CLASS};


  TListNode   = CLASS(TBaseNode)
  PRIVATE
    FoLines   : TStringList;
  PROTECTED
    FUNCTION    GetIsWord:BOOLEAN; OVERRIDE;
    FUNCTION    GetStr:STRING; OVERRIDE;
  PUBLIC
    PROPERTY    Lines:TStringList READ FoLines WRITE FoLines;
  PUBLIC
    CONSTRUCTOR Create;
    DESTRUCTOR  Destroy; OVERRIDE;
  END {CLASS};


  TCommentNode= CLASS(TListNode)
  PUBLIC
    PROPERTY    Comment:TStringList READ FoLines WRITE FoLines;
  END {CLASS};


  TFormatNode = CLASS(TListNode);


  TTokenNode  = CLASS(TBaseNode)
  PRIVATE
    FeToken   : TTokenKinds;
  PROTECTED
    FUNCTION    GetIsWord:BOOLEAN; OVERRIDE;
    FUNCTION    GetStr:STRING; OVERRIDE;
    PROCEDURE   SetStr(asValue:STRING); VIRTUAL;
    PROCEDURE   SetToken(aeSymbol:TTokenKinds); VIRTUAL;
  END {CLASS};


  TSymbolNode = CLASS(TTokenNode)
  PROTECTED
    PROCEDURE   SetToken(aeSymbol:TTokenKinds); OVERRIDE;
  PUBLIC
    PROPERTY    Symbol :TTokenKinds READ FeToken WRITE SetToken;
  END {CLASS};


  TPasWordNode= CLASS(TTokenNode)
  PROTECTED
    PROCEDURE   SetToken(aeSymbol:TTokenKinds); OVERRIDE;
  PUBLIC
    PROPERTY    PWord : TTokenKinds READ FeToken WRITE SetToken;
  END {CLASS};


  TValueNode  = CLASS(TTokenNode)
  PRIVATE
    FsValue   : STRING;
  PROTECTED
    FUNCTION    GetStr:STRING; OVERRIDE;
  PUBLIC
    PROPERTY    Value : STRING READ FsValue WRITE SetStr;
  END {CLASS};


  TIdentNode  = CLASS(TValueNode)
  PROTECTED
    PROCEDURE   SetStr(asValue:STRING); OVERRIDE;
  END {CLASS};


  TLitString  = CLASS(TValueNode)
  PROTECTED
    PROCEDURE   SetStr(asValue:STRING); OVERRIDE;
  END {CLASS};


  TLitFloat   = CLASS(TValueNode)
  PROTECTED
    PROCEDURE   SetStr(asValue:STRING); OVERRIDE;
  END {CLASS};


  TLitInteger = CLASS(TValueNode)
  PROTECTED
    PROCEDURE   SetStr(asValue:STRING); OVERRIDE;
  END {CLASS};


  TKeyType    = (ktUndefined, ktConst,   ktType, ktVar, ktField, ktParamVal,
                 ktParamVar,  ktProgram, ktProc, ktFunc);
              {=================================================}
IMPLEMENTATION
USES
  SysUtils;

{}CONSTRUCTOR TBaseNode.Create;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    INHERITED Create;
    FoCBlock:= NIL;
    FiNext  := -1;
{}END {Create};




{}DESTRUCTOR  TBaseNode.Destroy;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    IF HasBlock THEN BEGIN
      FoCBlock.Free;
    END {IF};

    INHERITED Destroy;
{}END {Destroy};




{}FUNCTION    TBaseNode.GetHasBlock:BOOLEAN;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    result := Assigned(FoCBlock);
{}END {GetHasBlock};




{}CONSTRUCTOR TListNode.Create;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    INHERITED Create;
    FoLines := TStringList.Create;
{}END {Create};




{}DESTRUCTOR  TListNode.Destroy;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    FoLines.Free;
    INHERITED Destroy;
{}END {Destroy};




{}FUNCTION    TListNode.GetIsWord:BOOLEAN;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    result := FALSE;
{}END {GetIsWord};




{}FUNCTION    TListNode.GetStr:STRING;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
{}END {GetStr};




{}FUNCTION    TTokenNode.GetIsWord:BOOLEAN;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    result := FeToken IN [rwAnd..swOn,tkIdentifier..tkLiteralString,tkFormattedBlock];
{}END {GetIsWord};




{}FUNCTION    TTokenNode.GetStr:STRING;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    result := TokenIs(FeToken);
{}END {GetStr};




{}PROCEDURE   TTokenNode.SetStr(asValue:STRING);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
{}END {SetStr};




{}PROCEDURE   TTokenNode.SetToken(aeSymbol:TTokenKinds);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    FeToken := aeSymbol;
{}END {SetToken};




{}PROCEDURE   TSymbolNode.SetToken(aeSymbol:TTokenKinds);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    IF aeSymbol IN [symAmpersand..symSlash] THEN BEGIN
      INHERITED SetToken(aeSymbol);
    END ELSE BEGIN
      RAISE Exception.Create(TokenIs(aeSymbol)+' is not a named symbol.');
    END {IF};
{}END {SetToken};




{}FUNCTION    TValueNode.GetStr:STRING;
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    result := FsValue;
{}END {GetStr};




{}PROCEDURE   TPasWordNode.SetToken(aeSymbol:TTokenKinds);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    IF aeSymbol IN [rwAnd..swOn] THEN BEGIN
      INHERITED SetToken(aeSymbol);
    END ELSE BEGIN
      RAISE Exception.Create(TokenIs(aeSymbol)+' is not a Reserved word, a Directive or a Special Word.');
    END {IF};
{}END {SetToken};




{}PROCEDURE   TIdentNode.SetStr(asValue:STRING);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    SetToken(tkIdentifier);
    FsValue := asValue;
{}END {SetStr};




{}PROCEDURE   TLitString.SetStr(asValue:STRING);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    SetToken(tkLiteralString);
    FsValue := asValue;
{}END {SetStr};




{}PROCEDURE   TLitFloat.SetStr(asValue:STRING);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    SetToken(tkLiteralFloat);
    FsValue := asValue;
{}END {SetStr};




{}PROCEDURE   TLitInteger.SetStr(asValue:STRING);
{+M
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  BEGIN
    SetToken(tkLiteralInteger);
    FsValue := asValue;
{}END {SetStr};



END {KSSsymbols}.
