{.delphi}
UNIT KSString;
{+H
---------------------------------------------------------------------------
  File        - KSSTRING.PAS

  Copyright    1999..2000 Keith S. Brown.

  Remarks     - This unit supplies string manipulation routines.
                (Delphi's SysUtils has some of the same functions)

  Assumptions - [1] The compiler has a string type which is defined as a
                sequence of characters with a dynamic length attribute
                (depending on the actual character count during program
                execution) and a constant size attribute from 1 to 255.
                A string-type declared without a size attribute is given
                the default size attribute of 255.  The length attribute
                is returned by a standard function Length().

  string type --> ( STRING ) --+---------------------------------------+-->
                               |                                       |
                               +-->( [ )-->[unsigned integer]-->( ] )--+

                Closest VAX Pascal syntax is "VARYING[n]OF CHAR".  Other
                Pascal compilers with a heritage derived from the UCSD
                family (if I remember correctly) implement a string type
                equivalent to the Turbo/Borland string-type.

                [2] The following sytem supplied routines are assumed to
                exist:
                   Function Length(s:STRING):INTEGER;
                     -- returns the dynamic length of a string

                   Procedure Delete(s:STRING; index,count:INTEGER);
                     -- deletes COUNT characters from S starting at INDEX.

                   Function Chr(n:INTEGER):CHAR;
                     -- returns a charcter with a specifed ordinal value.

                   Function Ord(x):INTEGER;
                     -- returns the ordinal number of an ordinal-type value.

                   Procedure Str(x; VAR s:STRING)
                     -- converts a numeric value to its string representation.
                        X is any numeric (integer-type or real-type) value.

                   Procedure Val(s:STRING; VAR v; VAR code:INTEGER);
                     -- converts a string to its numeric representation.
                        V is an integer-type or real-type variable.  If the
                        string is somehow invalid, the position of the
                        offending character is returned in CODE; otherwise
                        code is zero.


  Revised     - 1995.0317 (KSB) Rewrote routines for portability.
              - 1995.0329 (KSB) Changed set literals to set constants (for C-translation/portability)
              - 1995.0908 (KSB) Renamed from Pstring (developed for CELSS) to Wstr000.
              - 1997.1105 (KSB) (1.00) Added After, Before, DoubleCheck, Parse, Replace & ReplaceAll. Renamed to Wstr0100.
              - 1997.1106 (KSB) Added FloatStr (requires SysUtils).
              - 1997.1211 (KSB) Added IndefiniteArticle, TrimLeft & TrimRight.
              - 1998.0610 (KSB) Generalized to remove Delphi dependencies. Removed redundant functions. Added ByteToHexStr,
                HexStrToByte, ProperCase, IsUpCase, and PadLimit functions.
              - 1998.0611 (KSB) Added CenterStr function.
              - 1998.0831 (KSB) Added ParseFromTail function.
              - 1999.0613 (KSB) Changed name from WStr0100 to KSString.
              - 2000.08.03(KSB) Added LastChars function.
              - 2000.08.11(KSB) Added StrInsert function.
---------------------------------------------------------------------------}
{$IFDEF Win32}
  {$DEFINE Delphi}
{$ELSE}
  {$DEFINE TP}
{$ENDIF}

{$IFDEF TP}
  {$N+,E+}
{$ENDIF}
INTERFACE
TYPE
  TType       = (_NotANo,_real,_int);



{}FUNCTION  AddBackSlash(asPath,asFileName:STRING):STRING;
    { returns the complete file path with a backslash added after the path if needed.
    }
{}FUNCTION  After(asSource,asTarget:STRING):STRING;
    { returns the portion of asSource that follows asTarget in asSource.
    }
{}FUNCTION  Before(asSource,asTarget:STRING):STRING;
    { returns the portion of asSource that preceeds asTarget in asSource.
    }
{}FUNCTION  ByteToHexStr(aByte:BYTE):STRING;
    { returns two hexidecimal digits (as a string) for the passed byte value.
    }
{}FUNCTION  CenterStr(CONST aStr:STRING; aiWidth:INTEGER):STRING;
    { center aStr in a field aiWidth wide.
    }
{}FUNCTION  DoubleCheck(asSource:STRING; aChar:CHAR):STRING;
    { replace all multiple contiguous occurances of aChar in asSource with
      a single occurance of aChar.
    }
{}FUNCTION  FloatStr(aValue:EXTENDED; aiPrecision,aiDigits:INTEGER):STRING;
    { format a floating point value to be right justified in a field of
      aPrecision characters with aDigits digits to the right of the decimal.
    }
{}FUNCTION  HexStrToByte(aHexValue:STRING):BYTE;
    { returns a byte value for a string that consists of two hexidecimal
      digits.
    }
{}FUNCTION  IndefiniteArticle(aStr:STRING):STRING;
    { Adds a "a" or "an" prefix to the passed string.
    }
{}FUNCTION  InsertStr(asSource,aStr:STRING; aIndex:INTEGER):STRING;
    { Inserts asSource into aStr at the aIndex'th position.
    }
{}FUNCTION  IsDigit(aCh:CHAR):BOOLEAN;
    { is aCh one of "0".."9" ?
    }
{}FUNCTION  IsExpSym(aCh:CHAR):BOOLEAN;
    { is aCh "E" or "e" ?
    }
{}FUNCTION  IsNumberStr(aStr:STRING):TType;
    { determine if aStr is a properly formatted number.
    }
{}FUNCTION  IsPeriod(aCh:CHAR):BOOLEAN;
    { is aCh a "." ?
    }
{}FUNCTION  IsSign(aCh:CHAR):BOOLEAN;
    { is aCh a "+" or "-" ?
    }
{}FUNCTION  IsUpCase(aCh:CHAR):BOOLEAN;
    { returns true if the character is upper case.
    }
{}FUNCTION  IntToFmtStr(aiValue:LongINT; aiWidth:INTEGER):STRING;
    { like IntToStr but the result is right justified in a field aiWidth
      characters long.
    }
{}FUNCTION  IntToStr(aiValue:LongINT):STRING;
    { converts a integer-type expression aiValue to its string representation.
    }
{}FUNCTION  LastChars(asValue:STRING; aiChars:WORD):STRING;
    { return the last aiChars of asValue. If aiChars is greater than the length
      of asValue then all of asValue is returned, left padded with enough blanks
      such that the length is equal to aiChars.
    }
{}FUNCTION  LeftPadCh(aStr:STRING; aCh:CHAR; aiLen:INTEGER):STRING;
    { returns aStr padded on the left with aCh until it is
      aiLen characters long.
    }
{}FUNCTION  LowerCase(aStr:STRING):STRING;
    { converts the string aStr to all lower case characters.
    }
{}FUNCTION  Pad(aStr:STRING; aiLen:INTEGER):STRING;
    { returns aStr padded on the right with spaces until it is aiLen characters
      long.
    }
{}FUNCTION  PadCh(aStr:STRING; aCh:CHAR; aiLen:INTEGER):STRING;
    { returns aStr padded with aCh until it is aiLen characters long.
    }
{}FUNCTION  PadLimit(aStr:STRING; aiLen:INTEGER):STRING;
    { returns aStr padded on the right with spaces until it is aiLen characters
      long. However, if aStr is longer than aiLen, it is truncated to aiLen
      characters.
    }
{}FUNCTION  Parse(VAR asSource:STRING; asSeparator:STRING):STRING;
    { Treats asSource as a stream of tokens, separated by a string
      delimiter called asSeparator.  Each call to PARSE returns a
      single new token from the stream.  When the tokens are all
      used up, it continues to return null strings.
    }
{}FUNCTION  ParseFromTail(VAR asSource:STRING; asSeparator:STRING):STRING;
    { Same as Parse, but used in cases when you want to parse
      in Right to Left order instead of Left to Right order.
    }
{}FUNCTION  ProperCase(CONST asSource:STRING):STRING;
    { Capitalize the first character of each word in the string.
    }
{}FUNCTION  Rencode(aFloat:EXTENDED; aiPrecision:INTEGER):STRING;
    { special formatting for real values.
    }
{}FUNCTION  Replace(asSource,asSub,asNew:STRING):STRING;
    { replace the first occurance of asSub in asSource with asNew.
    }
{}FUNCTION  ReplaceAll(asSource,asSub,asNew:STRING):STRING;
    { replace all occurances of asSub in asSource with asNew.
    }
{}FUNCTION  StrInsert(CONST asMain,asSub:STRING; aiPos:INTEGER):STRING;
    { Insert asSub in asMain after the aiPosition in asMain.
    }
{}FUNCTION  StrToFloatDef(aStr:STRING; aDefault:DOUBLE):EXTENDED;
    { converts aStr to a floating point value.  If the string is not
      a valid number, the default value is returned.
    }
{}FUNCTION  StrToIntDef(aStr:STRING; aDefault:LongINT):LongINT;
    { converts aStr to an integer value.  If the string is not a valid
      number, the default value is returned.
    }
{}FUNCTION  Trim(aStr:STRING):STRING;
    { Removes leading and trailing spaces from a string.
    }
{}FUNCTION  TrimLeft(CONST aStr:STRING):STRING;
    { Removes leading spaces from a string.
    }
{}FUNCTION  TrimRight(CONST aStr:STRING):STRING;
    { Removes trailing spaces from a string.
    }
{}FUNCTION  UpperCase(CONST aStr:STRING):STRING;
    { return aStr converted to upper case.
    }
              {=================================================}
IMPLEMENTATION
{$IFDEF Delphi}
USES
  SysUtils;
{$ENDIF}



{}FUNCTION  AddBackSlash(asPath,asFileName:STRING):STRING;
    { returns the complete file path with a backslash added after the path if needed.
    }
  BEGIN
    IF Length(asPath) > 0 THEN BEGIN
      IF NOT (asPath[Length(asPath)] IN [':','\']) THEN BEGIN
        asPath := asPath + '\';
      END {IF};
    END {IF};

    result := asPath + asFileName;
{}END {AddBackSlash};




{}FUNCTION After(asSource,asTarget:STRING):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Returns all of the source string that follows (but does not
                include) the first occurance of the target string.
  Result type - string.
  Author      - Dick Pountain. Byte; Dec 1988; Pp.307-314
---------------------------------------------------------------------------}
  BEGIN
    IF Pos(asTarget,asSource) = 0 THEN
      After  := ''
    ELSE BEGIN
      After  := Copy(asSource,Pos(asTarget,asSource)+Length(asTarget),Length(asSource));
    END {IF};
{}END {After};




{}FUNCTION Before(asSource,asTarget:STRING):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Returns all of the source string up to but not including
                the first occurance of the target string.
  Result type - string.
  Author      - Dick Pountain. Byte; Dec 1988; Pp.307-314
---------------------------------------------------------------------------}
  BEGIN
    IF Pos(asTarget,asSource) = 0 THEN
      Before := asSource
    ELSE BEGIN
      Before := Copy(asSource,1,Pred(Pos(asTarget,asSource)));
    END {IF};
{}END {Before};




{}FUNCTION ByteToHexStr(aByte:BYTE):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Returns two hexidecimal digits (as a string) for the
                passed byte value.
---------------------------------------------------------------------------}
{$IFDEF TP}
  CONST
    cHexDigits= '0123456789ABCDEF';
  VAR
    ls        : STRING[2];
    lsHD      : STRING[16];
  BEGIN
    ls := '..';
    lsHD := cHexDigits;
    ls[2] := lsHD[Succ(aByte AND $0F)];
    ls[1] := lsHD[Succ(aByte SHR $04)];
    ByteToHexStr := ls;
{$ELSE}
  BEGIN
    result := SysUtils.IntToHex(aByte,2);
{$ENDIF}
{}END {ByteToHexStr};




{}FUNCTION  CenterStr(CONST aStr:STRING; aiWidth:INTEGER):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - center aStr in a field aiWidth wide.
---------------------------------------------------------------------------}
  VAR
    ls        : STRING;
  BEGIN
    ls := aStr;
    WHILE Length(ls) < aiWidth DO BEGIN
      ls := ' ' + ls + ' ';
    END {WHILE};

    IF Length(ls) > aiWidth THEN BEGIN
      ls := Copy(ls,1,aiWidth);
    END {IF};

    CenterStr := ls;
{}END {CenterStr};




{}FUNCTION DoubleCheck(asSource:STRING; aChar:CHAR):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Removes all multiple separators from the input string asSource.
  Result type - string.
  Author      - S. Balch.  Byte; Apr 1989; Pp.40
  Revised     - 1991.0903 (KSB) Added check for aChar+aChar to prevent aChar
                from being appended if aChar+aChar is not found.
              - 1998.0610 (KSB) Fixed case where the repeated characters are
                the last characters in the string.
---------------------------------------------------------------------------}
  BEGIN
    IF Pos(aChar+aChar,asSource) > 0 THEN BEGIN
      REPEAT
        asSource := Before(asSource,aChar+aChar)+aChar+After(asSource,aChar+aChar);
{      UNTIL After(asSource,aChar+aChar) = '';}
      UNTIL Pos(aChar+aChar,asSource) = 0;
    END {IF};

    DoubleCheck := asSource;
{}END {DoubleCheck};




{}FUNCTION  FloatStr(aValue:EXTENDED; aiPrecision,aiDigits:INTEGER):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - format a floating point value to be right justified in a
                field of aiPrecision characters with aiDigits digits to the
                right of the decimal.
---------------------------------------------------------------------------}
{}{}FUNCTION  FloatToFmtStr:STRING;
    VAR
      ls      : STRING;
    BEGIN
      {$IFDEF Delphi}
      ls := FloatToStrF(aValue,ffFixed,aiPrecision,aiDigits);
      {$ELSE}
      Str(aValue:aiPrecision:aiDigits,ls);
      {$ENDIF}
      FloatToFmtStr := ls;
{}{}END {FloatToFmtStr};


  VAR
    ls        : STRING;
  BEGIN
    ls := FloatToFmtStr;
    WHILE Length(ls) < aiPrecision DO BEGIN
      ls := ' ' + ls;
    END {WHILE};

    FloatStr := ls;
{}END {FloatStr};




{}FUNCTION  HexStrToByte(aHexValue:STRING):BYTE;
{+H
---------------------------------------------------------------------------
  Purpose     - returns a byte value for a string that consists of two
                hexidecimal digits.
---------------------------------------------------------------------------}
  VAR
    li        : BYTE;
    liCode    : INTEGER;
    ls        : STRING[2];
  BEGIN
    ls := aHexValue;
    System.Val('$'+ls,li,liCode);
    HexStrToByte := li;
{}END {HexStrToByte};




{}FUNCTION  IndefiniteArticle(aStr:STRING):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Add "a " or "an " to the beginning of aStr, if aStr is not
                null.
  Revised     - 1997.1211 (KSB) Wrote initial version.
---------------------------------------------------------------------------}
  VAR
    lbHasVowelSound: BOOLEAN;
    ls        : STRING;
    lsResult  : STRING;
  BEGIN
    lsResult := aStr;
    ls := UpperCase(Trim(aStr));
    IF (Length(ls) > 0) THEN BEGIN
      lbHasVowelSound := ls[1] IN ['A','E','I','O','U'];
      IF Copy(ls,1,3) = 'ONE' THEN BEGIN
        lbHasVowelSound := FALSE;
      END ELSE
      IF Copy(ls,1,4) = 'HOUR' THEN BEGIN
        lbHasVowelSound := TRUE;
      END ELSE
      IF Copy(ls,1,4) = 'UNIT' THEN BEGIN
        lbHasVowelSound := FALSE;
      END ELSE
      IF Copy(ls,1,5) = 'EULOG' THEN BEGIN {eulogy}
        lbHasVowelSound := FALSE;
      END {IF};

      IF lbHasVowelSound THEN BEGIN
        lsResult := 'an ';
      END ELSE BEGIN
        lsResult := 'a ';
      END {IF};

      lsResult := lsResult + TrimLeft(aStr);
    END {IF};

    IndefiniteArticle := lsResult;
{}END {IndefiniteArticle};




{}FUNCTION InsertStr(asSource,aStr:STRING; aIndex:INTEGER):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Inserts asSource into aStr at the aIndex'th position.
---------------------------------------------------------------------------}
  BEGIN
    System.Insert(asSource,aStr,aIndex);
    InsertStr := aStr;
{}END {InsertStr};





{}FUNCTION IsDigit(aCh:CHAR):BOOLEAN;
{+H
---------------------------------------------------------------------------
  Purpose     - determine if aCh is one of "0".."9" ?
---------------------------------------------------------------------------}
  CONST
    csDigits  = ['0'..'9'];
  BEGIN
    IsDigit := aCh IN csDigits;
{}END {IsDigit};




{}FUNCTION IsExpSym(aCh:CHAR):BOOLEAN;
{+H
---------------------------------------------------------------------------
  Purpose     - determine if aCh is "E" or "e" " ?
---------------------------------------------------------------------------}
  BEGIN
    IsExpSym := (aCh = 'e') OR (aCh = 'E');
{}END {IsExpSym};




{}FUNCTION  IsNumberStr(aStr:STRING):TType;
{+H
---------------------------------------------------------------------------
  Purpose     - determine if aStr is a properly formatted number.
---------------------------------------------------------------------------}
  VAR
    lbDecimal : BOOLEAN;
    lbNumber  : BOOLEAN;
    lbSign    : BOOLEAN;
  BEGIN
    lbDecimal := FALSE;
    lbNumber  := FALSE;
    lbSign    := FALSE;

    WHILE (Length(aStr) > 0) DO BEGIN
      IF IsSign(aStr[1]) THEN BEGIN
        IF lbSign THEN BEGIN
          IsNumberStr := _NotANo;      {cannot have two signs}
          Exit;
        END ELSE BEGIN
          lbSign := TRUE;
        END {IF};
      END ELSE
      IF IsDigit(aStr[1]) THEN BEGIN
        lbNumber := TRUE;
      END ELSE
      IF lbNumber THEN BEGIN
        IF (aStr[1] = '.') THEN BEGIN
          lbDecimal := TRUE;
        END ELSE
        IF IsExpSym(aStr[1]) THEN BEGIN
          lbSign := FALSE;
        END ELSE BEGIN
          IsNumberStr := _NotANo;      {a digit sequence followed by}
          Exit;                        {other than a digit, a decimal}
        END {IF};                      {or a exponent is not a number}

        lbNumber := FALSE;
      END ELSE BEGIN
        IsNumberStr := _NotANo;        {a sign must be followed by a}
        Exit;                          {digit sequence to be a valid}
      END {IF};                        {number}

      Delete(aStr,1,1);
    END {WHILE};

    IF lbNumber THEN BEGIN
      IF lbDecimal THEN BEGIN
        IsNumberStr := _real;         {digits and decimal found -> float}
      END ELSE BEGIN
        IsNumberStr := _int;          {digits found -> integer}
      END {IF};
    END ELSE BEGIN
      IsNumberStr := _NotANo;
    END {IF};
{}END {IsNumberStr};




{}FUNCTION IsPeriod(aCh:CHAR):BOOLEAN;
{+H
---------------------------------------------------------------------------
  Purpose     - is aCh a "." ?
---------------------------------------------------------------------------}
  BEGIN
    IsPeriod := aCh = '.';
{}END {IsPeriod};




{}FUNCTION IsSign(aCh:CHAR):BOOLEAN;
{+H
---------------------------------------------------------------------------
  Purpose     - is aCh a "+" or "-" ?
---------------------------------------------------------------------------}
  BEGIN
    IsSign := (aCh = '-') OR (aCh = '+');
{}END {IsSign};




{}FUNCTION  IsUpCase(aCh:CHAR):BOOLEAN;
{+H
---------------------------------------------------------------------------
  Purpose     - Returns true if aCh is an upper case character.
---------------------------------------------------------------------------}
  BEGIN
    IsUpCase := aCh = UpCase(aCh);
{}END {IsUpCase};




{}FUNCTION IntToFmtStr(aiValue:LongINT; aiWidth:INTEGER):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - like IntToStr but the result is right justified in a field
                aiWidth characters long.
---------------------------------------------------------------------------}
  VAR
    ls        : STRING;
  BEGIN
    System.Str(aiValue:aiWidth,ls);
    IntToFmtStr := ls;
{}END {IntToFmtStr};





{}FUNCTION IntToStr(aiValue:LongINT):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - converts a integer-type expression aiValue to its string
                representation.
---------------------------------------------------------------------------}
{$IFDEF TP}
  VAR
    ls        : STRING;
  BEGIN
    Str(aiValue,ls);
    IntToStr := ls;
{$ELSE}
  BEGIN
    result := SysUtils.IntToStr(aiValue);
{$ENDIF}
{}END {IntToStr};




{}FUNCTION    LastChars(asValue:STRING; aiChars:WORD):STRING;
{+O
---------------------------------------------------------------------------
  Purpose     - return the last aiChars of asValue. If aiChars is greater
                than the length of asValue then all of asValue is returned,
                left padded with enough blanks such that the length is equal
                to aiChars.
---------------------------------------------------------------------------}
  VAR
    li        : INTEGER;
  BEGIN
    result := '';
    FOR li := Length(asValue) DOWNTO Length(asValue)-aiChars+1 DO BEGIN
      IF li > 0 THEN BEGIN
        result := asValue[li] + result;
      END ELSE BEGIN
        result := ' ' + result;
      END {IF};
    END {FOR};
{}END {LastChars};




{}FUNCTION    LeftPadCh(aStr:STRING; aCh:CHAR; aiLen:INTEGER):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - returns aStr padded on the left with aCh until it is
                aiLen characters long.
---------------------------------------------------------------------------}
  BEGIN
    WHILE Length(aStr) < aiLen DO BEGIN
      aStr := aCh + aStr;
    END {WHILE};

    LeftPadCh := aStr;
{}END {LeftPadCh};




{}FUNCTION  LowerCase(aStr:STRING):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - converts the string aStr to all lower case characters.
---------------------------------------------------------------------------}
{$IFDEF TP}
  CONST
    cUpperCase= ['A'..'Z'];
  VAR
    li        : INTEGER;
  BEGIN
    FOR li := 1 TO Length(aStr) DO BEGIN
      IF aStr[li] IN cUpperCase THEN BEGIN
        aStr[li] := Chr( Ord(aStr[li])+32 );
      END {IF};
    END {FOR};

    LowerCase := aStr;
{$ELSE}
  BEGIN
    result := SysUtils.LowerCase(aStr);
{$ENDIF}
{}END {LowerCase};




{}FUNCTION Pad(aStr:STRING; aiLen:INTEGER):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - returns aStr padded on the right with spaces until it is
                aiLen characters long.
---------------------------------------------------------------------------}
  BEGIN
    WHILE Length(aStr) < aiLen DO BEGIN
      aStr := aStr + ' ';
    END {WHILE};

    Pad := aStr;
{}END {Pad};




{}FUNCTION  PadCh(aStr:STRING; aCh:CHAR; aiLen:INTEGER):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - returns aStr padded with aCh until it is aiLen characters
                long.
---------------------------------------------------------------------------}
  BEGIN
    WHILE Length(aStr) < aiLen DO BEGIN
      aStr := aStr + aCh;
    END {WHILE};

    PadCh := aStr;
{}END {PadCh};




{}FUNCTION  PadLimit(aStr:STRING; aiLen:INTEGER):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - returns aStr padded on the right with spaces until it is
                aiLen characters long. However, if aStr is longer than aiLen,
                it is truncated to aiLen characters.
---------------------------------------------------------------------------}
  VAR
    ls        : STRING;
  BEGIN
    ls := Pad(aStr,aiLen);
    IF Length(ls) > aiLen THEN BEGIN
      ls := Copy(ls,1,aiLen);
    END {IF};

    PadLimit := ls;
{}END {PadLimit};




{}FUNCTION    Parse(VAR asSource:STRING; asSeparator:STRING):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Treats asSource as a stream of tokens, separated by a string
                delimiter called asSeparator.  Each call to PARSE returns a
                single new token from the stream.  When the tokens are all
                used up, it continues to return null strings.
  Warning     - This function modifies its arguments, ie., it lacks idempotency.
  Result type - string.
  Author      - Dick Pountain. Byte; Dec 1988; Pp.307-314
---------------------------------------------------------------------------}
  BEGIN
    Parse   := Before(asSource,asSeparator);
    asSource:= After(asSource,asSeparator);
{}END {Parse};




{}FUNCTION    ParseFromTail(VAR asSource:STRING; asSeparator:STRING):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Same as Parse, but used when you want to parse in
                Right to Left order instead of Left to Right order.
---------------------------------------------------------------------------}
  BEGIN
    ParseFromTail := After(asSource,asSeparator);
    asSource:= Before(asSource,asSeparator);
{}END {ParseFromTail};




{}FUNCTION  ProperCase(CONST asSource:STRING):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Capitalize the first character of each word in the string.
---------------------------------------------------------------------------}
  CONST
    WhiteSpace: SET OF Char = [#$00..#$40,#$5B..#$60,#$7B..#$7F];
  VAR
    li        : INTEGER;
    ls        : STRING;
  BEGIN
    ls := ' '+LowerCase(asSource);
    FOR li := 2 TO Length(ls) DO BEGIN
      IF (ls[li-1] IN WhiteSpace) AND NOT(ls[li] IN WhiteSpace) THEN BEGIN
        ls[li] := UpCase(ls[li]);
      END {IF};
    END {FOR};

    Delete(ls,1,1);
    ProperCase := ls;
{}END {ProperCase};




{}FUNCTION  Rencode(aFloat:EXTENDED; aiPrecision:INTEGER):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Convert a real value to a string.
  Author      - adapted from code by K. R. Pierce
  Revised     - 1995.0321 (KSB) Converted to a function.
---------------------------------------------------------------------------}
{}{}FUNCTION PwrOfTen (aN : INTEGER) : EXTENDED;
{---------------------------------------------------------------------------
  Purpose     - Return 10 raised to the power aN
----------------------------------------------------------------------------}
    VAR
      li      : INTEGER;
      lf      : EXTENDED;
    BEGIN
      lf := 1;
      FOR li := 1 TO Abs(aN) DO BEGIN
        lf := lf*10.0;
      END {FOR};

      IF aN >= 0 THEN
        PwrOfTen := lf
      ELSE BEGIN
        PwrOfTen := 1.0/lf
      END {IF};
{}{}END {PwrOfTen};


  CONST
    csDigit   = '0123456789';{ Hold digit charactrs for encoding }
  VAR
    li        : INTEGER;     { Loop control }
    liScale   : INTEGER;     { Power of 10 multiplier needed to bring aFloat in range 1..10 }
    lbIsNegative   : BOOLEAN;{ Flag for sign of aFloat  }
    lbNeedsExpnt   : BOOLEAN;{ Exponent required}
    ls        : STRING;
    lsExponent: STRING;      { Encoding of integer exponent }
  BEGIN
    IF aFloat=0.0 THEN BEGIN { we're done}
      ls := '0'
    END ELSE BEGIN                     { --- aFloat is nonzero. }
      IF aFloat < 0 THEN BEGIN         { --- check for sign}
        aFloat := - aFloat;
        lbIsNegative := TRUE
      END ELSE BEGIN
        lbIsNegative := FALSE;
      END {IF};

      liScale := 0;                    { --- find scale so that 1 <= aFloat*10 >(-Scale^ 10 }
      IF aFloat >= 10 THEN BEGIN
        REPEAT
          liScale := liScale + 1;
          aFloat := aFloat / 10.0
        UNTIL aFloat < 10
      END ELSE BEGIN
        WHILE aFloat < 1 DO BEGIN
          liScale := liScale - 1;
          aFloat := aFloat * 10
        END {WHILE};
      END {IF};

      { --- round least precise digit --- }
      aFloat := aFloat + 5 /PwrOfTen(aiPrecision);


      IF aFloat >= 10.0 THEN BEGIN     { --- adjust if rounding exceeds 10}
        aFloat:= 1.0;
        Inc(liScale);
      END {IF};


      ls := '';                        { --- get significant digits into ls}
      FOR li := 1 TO aiPrecision DO BEGIN
        ls := concat(ls, Copy(csDigit, Trunc(aFloat) + 1, 1));
        aFloat := 10 * (aFloat - Trunc(aFloat));
      END {FOR};


      lbNeedsExpnt :=  FALSE;          { --- Put in decimal, and exponent if necessary}
      IF (0 <= liScale) AND (liScale <= aiPrecision - 2) THEN
        ls := InsertStr('.',ls , liScale + 2)
      ELSE
      IF liScale = -1 THEN
        ls := Concat( '0.', ls)
      ELSE
      IF liScale = aiPrecision-1 THEN
        ls := Concat(ls, '.0')
      ELSE BEGIN                       {'E' notation is necessary }
        IF aiPrecision > 1 THEN BEGIN
          ls := InsertStr('.',ls,2);
        END {IF};

        lbNeedsExpnt := TRUE
      END {IF};

      { --- eliminate trailing zeros right of decimal point
            and decimal point if fractional part all zeros}
      WHILE ls[Length(ls)] = '0' DO BEGIN
        ls := Copy(ls, 1, Length(ls) - 1);
      END {WHILE};

      IF ls[Length(ls)] = '.' THEN BEGIN
        ls := Copy(ls, 1, Length(ls) - 1);
      END {IF};

      IF lbNeedsExpnt THEN BEGIN       { --- Put in exponent if needed}
        lsExponent := Trim(IntToStr(liScale));
        ls := Concat(ls, 'E');
        ls := Concat(ls, lsExponent)
      END {IF};


      IF lbIsNegative THEN BEGIN       { --- adjust sign --- }
        ls := concat('-', ls);
      END {IF};
    END {IF};

    RenCode := ls;
{}END {RenCode};




{}FUNCTION  Replace(asSource,asSub,asNew:STRING):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Replace the first occurance of asSub found in asSource with asNew
  Result type - string.
---------------------------------------------------------------------------}
  BEGIN
    IF Pos(asSub,asSource)>0 THEN
      Replace := Before(asSource,asSub)+asNew+After(asSource,asSub)
    ELSE BEGIN
      Replace := asSource;
    END {IF};
{}END {Replace};




{}FUNCTION  ReplaceAll(asSource,asSub,asNew:STRING):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Replace all occurances of asSub found in asSource with asNew.
  Result type - string.
---------------------------------------------------------------------------}
  BEGIN
    WHILE Pos(asSub,asSource)>0 DO BEGIN
      asSource := Replace(asSource,asSub,asNew);
    END {WHILE};

    ReplaceAll := asSource;
{}END {ReplaceAll};




{}FUNCTION    StrInsert(CONST asMain,asSub:STRING; aiPos:INTEGER):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Insert asSub in asMain after the aiPosition in asMain.
                If aiPos is 0 the result is the same as asSub + asMain
                If aiPos is negative, then Abs(aiPos) blanks are inserted
                  after asSub and before asMain
                If aiPos is positive, but less than or equal to the length
                  of asMain, then asSub is inserted into asMain
                If aiPos is positive, but larger than the length of asMain,
                  then enough blanks are appended to asMain such that asSub
                  is inserted at the correct location.
---------------------------------------------------------------------------}
  BEGIN
    IF aiPos = 0 THEN BEGIN
      result := asSub + asMain;
    END ELSE
    IF aiPos < 0 THEN BEGIN
      result := asSub + StringOfChar(' ',Abs(aiPos)) + asMain;
    END ELSE
    IF aiPos < Length(asMain) THEN BEGIN
      result := Copy(asMain,1,aiPos) + asSub + Copy(asMain,aiPos+1,Length(asMain)-aiPos);
    END ELSE BEGIN
      result := asMain + StringOfChar(' ',aiPos-Length(asMain)) + asSub;
    END {IF};
{}END {StrInsert};




{}FUNCTION  StrToFloatDef(aStr:STRING; aDefault:DOUBLE):EXTENDED;
{+H
---------------------------------------------------------------------------
  Purpose     - converts aStr to a floating point value.  If the string is
                not a valid number, the default value is returned.
---------------------------------------------------------------------------}
{$IFDEF TP}
  VAR
    lf        : EXTENDED;
    liCode    : INTEGER;
  BEGIN
    aStr := Trim(aStr);
    System.Val(aStr,lf,liCode);
    IF liCode <> 0 THEN BEGIN
      lf := aDefault;
    END {IF};

    StrToFloatDef := lf;
{$ELSE}
  BEGIN
    TRY
      result := StrToFloat(aStr);
    EXCEPT
      ON EConvertError DO BEGIN
        result := aDefault;
      END {ON};
    END {TRY};
{$ENDIF}
{}END {StrToFloatDef};




{}FUNCTION  StrToIntDef(aStr:STRING; aDefault:LongINT):LongINT;
{+H
---------------------------------------------------------------------------
  Purpose     - converts aStr to an integer value.  If the string is not a
                valid number, the default value is returned.
---------------------------------------------------------------------------}
{$IFDEF TP}
  VAR
    li        : LongINT;
    liCode    : INTEGER;
  BEGIN
    aStr := Trim(aStr);
    System.Val(aStr,li,liCode);
    IF liCode <> 0 THEN BEGIN
      li := aDefault;
    END {IF};

    StrToIntDef := li;
{$ELSE}
  BEGIN
    result := SysUtils.StrToIntDef(aStr,aDefault);
{$ENDIF}
{}END {StrToIntDef};




CONST
  cTrimable  = [#0..#32,#127..#255];


{}FUNCTION  Trim(aStr:STRING):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Removes leading and trailing spaces from a string.
---------------------------------------------------------------------------}
  BEGIN
    Trim := TrimLeft(TrimRight(aStr));
{}END {Trim};




{}FUNCTION  TrimLeft(CONST aStr:STRING):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Remove leading non-printing characters from a string.
  Revised     - 1997.1211 (KSB) Wrote initial version.
---------------------------------------------------------------------------}
  VAR
    ls        : STRING;
  BEGIN
    ls := aStr;
    WHILE (Length(ls) > 0) AND (ls[1] IN cTrimable) DO BEGIN
      Delete(ls,1,1);
    END {WHILE};

    TrimLeft := ls;
{}END {TrimLeft};




{}FUNCTION  TrimRight(CONST aStr:STRING):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - Remove trailing non-printing characters from a string.
  Revised     - 1997.1211 (KSB) Wrote initial version.
---------------------------------------------------------------------------}
  VAR
    ls        : STRING;
  BEGIN
    ls := aStr;
    WHILE (Length(ls) > 0) AND (ls[Length(ls)] IN cTrimable) DO BEGIN
      Delete(ls,Length(ls),1);
    END {WHILE};

    TrimRight := ls;
{}END {TrimRight};




{}FUNCTION  UpperCase(CONST aStr:STRING):STRING;
{+H
---------------------------------------------------------------------------
  Purpose     - return aStr converted to upper case.
---------------------------------------------------------------------------}
{$IFDEF TP}
  VAR
    li        : INTEGER;
    ls        : STRING;
  BEGIN
    ls := aStr;
    FOR li := 1 TO Length(aStr) DO BEGIN
      ls[li] := UpCase(aStr[li]);
    END {FOR};

    UpperCase := ls;
{$ELSE}
  BEGIN
    result := SysUtils.UpperCase(aStr);
{$ENDIF}
{}END {UpperCase};


END {KSSTRING}.