UNIT LstGlobal;
{+F
---------------------------------------------------------------------------
  File        - LSTGLOBAL.PAS
  Purpose     - Global declarations & routines.
  Author      - Keith S. Brown
  Copyright    2000 by KSS  http://www.ghg.net/chairouchu
  Revised     - 2000.08.23(KSB) Created original.
---------------------------------------------------------------------------}
INTERFACE
USES
  Controls;

TYPE
  TEditorIs   = (eiWordPad,eiNotePad,eiOther);

VAR
  gbUseRTL    : BOOLEAN;
  gbUseSearchP: BOOLEAN;
  gbUseVCL    : BOOLEAN;
  geEditorIs  : TEditorIs;
  gsDelphi_RTL: STRING;
  gsDelphi_VCL: STRING;
  gsEditorPath: STRING;
  gsReportFile: STRING;
  gsSearchPath: STRING;

{}PROCEDURE   ReadConfiguration;
{}PROCEDURE   ReadControlPos(aoForm:TControl);
{}PROCEDURE   SaveConfiguration;
{}PROCEDURE   SaveControlPos(aoForm:TControl);
              {=================================================}
IMPLEMENTATION
USES
  SysUtils,
  Forms,
  IniFiles;

CONST
  csLastRTL   = 'RTL';
  csLastSearch= 'Search Paths';
  csLastSectn = 'LAST';
  csLastVCL   = 'VCL';
  csLastView  = 'Viewer';
  csLastVPath = 'Viewer Path';
  csUseRTL    = 'Use RTL';
  csUseSearchP= 'Use Search Path';
  csUseVCL    = 'Use VCL';


{}PROCEDURE   ReadConfiguration;
{+M
---------------------------------------------------------------------------
  Purpose     - Read global values from ini file.
  Revised     - 2000.08.24(KSB) Added gbUseRTL & gbUseVCL
---------------------------------------------------------------------------}
  VAR
    liEditor  : INTEGER;
    loINI     : TIniFile;
  BEGIN
    loINI := TIniFile.Create(ChangeFileExt(application.ExeName,'.INI'));
    TRY
      liEditor     := loINI.ReadInteger(csLastSectn,csLastView,  0);
      gsDelphi_RTL := loINI.ReadString (csLastSectn,csLastRTL,   '');
      gsDelphi_VCL := loINI.ReadString (csLastSectn,csLastVCL,   '');
      gsEditorPath := loINI.ReadString (csLastSectn,csLastVPath, '');
      gsSearchPath := loINI.ReadString (csLastSectn,csLastSearch,'');
      gbUseRTL     := loINI.ReadBool   (csLastSectn,csUseRTL,    FALSE);
      gbUseSearchP := loINI.ReadBool   (csLastSectn,csUseSearchP,FALSE);
      gbUseVCL     := loINI.ReadBool   (csLastSectn,csUseVCL,    FALSE);
      CASE liEditor OF
        00 : geEditorIs := eiWordPad;
        01 : geEditorIs := eiNotePad;
        ELSE geEditorIs := eiOther;
      END {CASE};
    FINALLY
      FreeAndNil(loINI);
    END {TRY};
{}END {ReadConfiguration};




{}PROCEDURE   ReadControlPos(aoForm:TControl);
{+M
---------------------------------------------------------------------------
  Purpose     - Read the position of a control.
---------------------------------------------------------------------------}
  VAR
    loINI     : TIniFile;
  BEGIN
    loINI := TIniFile.Create(ChangeFileExt(application.ExeName,'.INI'));
    TRY
      WITH aoForm DO BEGIN
        Top     := loINI.ReadInteger(Name,    'Top',     Top);
        Left    := loINI.ReadInteger(Name,    'Left',    Left);
        Height  := loINI.ReadInteger(Name,    'Height',  Height);
        Width   := loINI.ReadInteger(Name,    'Width',   Width);
      END {WITH};
    FINALLY
      FreeAndNil(loINI);
    END {TRY};
{}END {ReadControlPos};




{}PROCEDURE   SaveConfiguration;
{+M
---------------------------------------------------------------------------
  Purpose     - Svae global values to ini file.
  Revised     - 2000.08.24(KSB) Added gbUseRTL & gbUseVCL
---------------------------------------------------------------------------}
  VAR
    liEditor  : INTEGER;
    loINI     : TIniFile;
  BEGIN
    loINI := TIniFile.Create(ChangeFileExt(application.ExeName,'.INI'));
    TRY
      CASE geEditorIs OF
        eiWordPad : liEditor := 0;
        eiNotePad : liEditor := 1;
        ELSE        liEditor := 2;
      END {CASE};

      loINI.WriteInteger(csLastSectn, csLastView,  liEditor);
      loINI.WriteString (csLastSectn, csLastRTL,   gsDelphi_RTL);
      loINI.WriteString (csLastSectn, csLastSearch,gsSearchPath);
      loINI.WriteString (csLastSectn, csLastVPath, gsEditorPath);
      loINI.WriteString (csLastSectn, csLastVCL,   gsDelphi_VCL);
      loINI.WriteBool   (csLastSectn, csUseRTL,    gbUseRTL);
      loINI.WriteBool   (csLastSectn, csUseSearchP,gbUseSearchP);
      loINI.WriteBool   (csLastSectn, csUseVCL,    gbUseVCL);
    FINALLY
      FreeAndNil(loINI);
    END {TRY};
{}END {SaveConfiguration};




{}PROCEDURE   SaveControlPos(aoForm:TControl);
{+M
---------------------------------------------------------------------------
  Purpose     - Save the position of a control.
---------------------------------------------------------------------------}
  VAR
    loINI     : TIniFile;
  BEGIN
    loINI := TIniFile.Create(ChangeFileExt(application.ExeName,'.INI'));
    TRY
      WITH aoForm DO BEGIN
        loINI.WriteInteger(Name,       'Top',     Top);
        loINI.WriteInteger(Name,       'Left',    Left);
        loINI.WriteInteger(Name,       'Height',  Height);
        loINI.WriteInteger(Name,       'Width',   Width);
      END {WITH};
    FINALLY
      FreeAndNil(loINI);
    END {TRY};
{}END {SaveControlPos};


INITIALIZATION
  gsReportFile := 'W32DelphiTool.TXT';
END {LstGlobal}.
