UNIT UnitDpnd;
{+F
---------------------------------------------------------------------------
  File        - UNITDPND.PAS
  Purpose     - List unit dependencies.
  Author      - Keith S. Brown
  Copyright    2000 by KSS. All rights reserved. http://www.ghg.net/chairouchu
  Revised     - 2000.08.22(KSB) extracted unit tree code from LST7 (1991).
---------------------------------------------------------------------------}
INTERFACE

{}PROCEDURE   Write_UnitDpnd(asMainFileName,asReportFileName:STRING);
              {=================================================}
IMPLEMENTATION
USES
  Classes,
  SysUtils,
  KDStruct,   {binary tree data structure}
  KSString,   {additional string routines}
  UnitTree;


TYPE
  TDpndNode   = CLASS(TTreeNode)
  PRIVATE
    FsName    : STRING;
    FoList    : TStringList;
  PUBLIC
    PROPERTY    List:TStringList READ FoList WRITE FoList;
    PROPERTY    Name:STRING      READ FsName WRITE FsName;
  PUBLIC
    CONSTRUCTOR Create(asKey:STRING);
    DESTRUCTOR  Destroy; OVERRIDE;
  END {CLASS};


  TDpndTree   = CLASS(TTree)
    FUNCTION    Compare(avKey1,avKey2:VARIANT):INTEGER; OVERRIDE;
    FUNCTION    GetKey(aoNode:TTreeNode):VARIANT; OVERRIDE;
  END {CLASS};


{}CONSTRUCTOR TDpndNode.Create(asKey:STRING);
  BEGIN
    FsName := asKey;
    FoList := TStringList.Create;
{}END {Create};



{}DESTRUCTOR  TDpndNode.Destroy;
  BEGIN
    FreeAndNil(FoList);
{}END {Destroy};



{}FUNCTION    TDpndTree.Compare(avKey1,avKey2:VARIANT):INTEGER;
  VAR
    ls1       : STRING;
    ls2       : STRING;
  BEGIN
    ls1 := UpperCase(avKey1);
    ls2 := UpperCase(avKey2);
    IF ls1 < ls2 THEN BEGIN
      result := -1
    END ELSE
    IF ls1 > ls2 THEN BEGIN
      result :=  1
    END ELSE BEGIN
      result :=  0;
    END {IF};
{}END {Compare};




{}FUNCTION    TDpndTree.GetKey(aoNode:TTreeNode):VARIANT;
  BEGIN
    result := TDpndNode(aoNode).Name;
{}END {GetKey};




VAR
  uiLineCount      : INTEGER;
  uiRefCount       : INTEGER;
  uiRefPerLine     : INTEGER;     { number of referenced line no / line}
  uoInvert         : TDpndTree;
  usMainFileName   : STRING;
  uTxt             : TEXTFILE;



{}FUNCTION    NewUnit(avKey:VARIANT):TTreeNode;
{+H
---------------------------------------------------------------------------
  Purpose     - Builds a new record for insertion into tree
---------------------------------------------------------------------------}
  BEGIN
    result := TDpndNode.Create(avKey);
{}END {NewUnit};




{}PROCEDURE   BuildInvTreeNode(aoSon,aoFather:TTreeNode);
{+H
---------------------------------------------------------------------------
  Purpose     - 
---------------------------------------------------------------------------}
  VAR
    loNode    : TTreeNode;
  BEGIN
    loNode := uoInvert.Search(TDpndNode(aoSon).Name,NewUnit);
    IF aoFather <> NIL THEN BEGIN
      WITH TDpndNode(loNode) DO BEGIN
        List.Append(TDpndNode(aoFather).Name);
      END {WITH};
    END {IF};
{}END {BuildInvTreeNode};




{}PROCEDURE   InvListDump(aiNo:INTEGER; asName:STRING);
{+H
---------------------------------------------------------------------------
  Purpose     -
  Revised     - 2000.08.24(KSB) Added line wrapping when long unit names occur.
---------------------------------------------------------------------------}
  CONST
    ciWidth   : INTEGER = 0;
  VAR
    li        : INTEGER;
  BEGIN
    IF aiNo = 0 THEN BEGIN
      ciWidth := 0;
    END {IF};

    IF (uiRefCount = uiRefPerLine) OR (ciWidth >= 87) THEN BEGIN
      WriteLn(uTxt);
      Write  (uTxt,' ':18);
      uiRefCount := 0;
      ciWidth := 0;
    END {IF};

    IF asName = '' THEN BEGIN
      Write(uTxt,' ------ ');
      Inc(ciWidth,8);
    END ELSE BEGIN
      IF Length(asName) > 9 THEN BEGIN
        li := ((Length(asName) DIV 10)+1)* 10;
      END ELSE BEGIN
        li := 10;
      END {IF};

      Write(uTxt,Pad(asName,li));
      Inc(ciWidth,li);
    END {IF};

    Inc(uiRefCount);
{}END {InvListDump};




{}PROCEDURE   InvTreeDump(aoNode:TTreeNode);
{+H
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  VAR
    li        : INTEGER;
    lsName    : STRING;
  BEGIN
    uiRefCount := 0;
    IF aoNode = NIL THEN BEGIN
      WriteLn(uTxt,' Inverted Tree Error ')
    END ELSE
    WITH TDpndNode(aoNode) DO BEGIN
      lsName := ExtractFileName(Name);
      IF Length(lsName) > 11 THEN BEGIN
        li := ((Length(lsName) DIV 10)+1)* 10;
      END ELSE BEGIN
        li := 10;
      END {IF};

      lsName := Pad(lsName,li+2);
      IF List.Count = 0 THEN BEGIN
        WriteLn(uTxt,uiLineCount:4,'  ',lsName,'*base unit*')
      END ELSE BEGIN
        Write  (uTxt,uiLineCount:4,'  ',lsName);
        FOR li := 0 TO List.Count-1 DO BEGIN
          InvListDump(li, List[li]);
        END {FOR};

        WriteLn(uTxt);
      END {IF};
    END {WITH};

    Inc(uiLineCount);
{}END {InvTreeDump};




{}PROCEDURE   Write_UnitDpnd(asMainFileName,asReportFileName:STRING);
{+H
---------------------------------------------------------------------------
  Purpose     - Write an inverted unit dependency list, ie.,
                if unit A is changed what units will it impact?
---------------------------------------------------------------------------}
{}{}PROCEDURE Inverted_Header;
    VAR
      li      : WORD;
    BEGIN
      Write_ReportHdr(uTxt);
      WriteLn(uTxt);
      WriteLn(uTxt,CenterStr('Unit Dependency Table for '+usMainFileName,ciCenteringWidth));
      WriteLn(uTxt);

      WriteLn(uTxt,' No.  __unit__    __is_used_by_');
      Write  (uTxt,' ---  --------    ');
      FOR li := 1 TO uiRefPerLine DO BEGIN
        Write(uTxt,'--------  ');
      END {FOR};

      WriteLn(uTxt);
{}{}END {Inverted_Header};


  BEGIN
    IF Assigned(goUnits) THEN BEGIN
      OpenTextFile(uTxt,asReportFileName);
      usMainFileName := asMainFileName;
      TRY
        uiRefPerLine := 9;
        uiLineCount  := 1;
        Inverted_Header;
        uoInvert := TDpndTree.Create;
        goUnits.Unravel(BuildInvTreeNode);
        uoInvert.ForEach(InvTreeDump);
        Flush(uTxt);
        FreeAndNil(uoInvert);
      FINALLY
        CloseFile(uTxt);
      END {TRY};
    END {IF};
{}END {Write_UnitDpnd};




END {UnitDpnd}.
